'''VideoCacheInotifyHandler - Update cache according to file system changes.'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import os
from pyinotify import ProcessEvent
from entertainerlib.backend.components.mediacache.video_cache import VideoCache

class VideoCacheInotifyHandler(ProcessEvent):
    """
    Update cache according to file system changes.'

    Object from this class handles inotify events for video cache.

    NOTE: VideoCache object is created in every method, because of SQLite
          doesn't allow cursor to be used in separate threads.
    """

    # File System Observer object
    __fs_observer = None

    def __init__(self, fs_observer):
        """
        Create a new VideoCacheInotifyHandler object, which handles file system
        changes. In other words, object from this class makes sure that cache
        is always up-to-date.
        """
        self.__fs_observer = fs_observer

    def process_IN_DELETE(self, event):
        """
        File deleted
        """
        video_cache = VideoCache()

        # File deleted - Remove file from cache
        if video_cache.isSupportedFormat(event.name):
            video_cache.removeFile(os.path.join(event.path, event.name))
        else:
            # Directory deleted - Remove directory from cache
            if video_cache.isDirectoryInCache(
                os.path.join(event.path, event.name)):
                self.__fs_observer.removeFolder(
                    os.path.join(event.path, event.name))
                video_cache.removeDirectory(
                    os.path.join(event.path, event.name))

    def process_IN_CREATE(self, event):
        """
        File created
        """
        video_cache = VideoCache()

        # Directory created - Add directory to cache
        if os.path.isdir(os.path.join(event.path, event.name)):
            self.__fs_observer.addFolder(
                os.path.join(event.path, event.name), "video")
            video_cache.addDirectory(os.path.join(event.path, event.name))

        # File created - Add file to cache
        else:
            if video_cache.isSupportedFormat(event.name):
                video_cache.addFile(os.path.join(event.path, event.name))

    def process_IN_MODIFY(self, event):
        """
        File or directory was modified.
        """
        video_cache = VideoCache()

        # Directory created - Add directory to cache
        if os.path.isdir(os.path.join(event.path, event.name)):
            self.__fs_observer.addFolder(
                os.path.join(event.path, event.name), "video")
            video_cache.updateDirectory(os.path.join(event.path, event.name))

        # File created - Add file to cache
        else:
            if video_cache.isSupportedFormat(event.name):
                video_cache.updateFile(os.path.join(event.path, event.name))

    def process_IN_MOVED_FROM(self, event):
        """
        File or directory is moved from observed. (same as delete in our
        point of view)
        """
        video_cache = VideoCache()
        self.process_IN_DELETE(event)

    def process_IN_MOVED_TO(self, event):
        """
        File or directory is moved in to observed. (same as create in our
        point of view)
        """
        self.process_IN_CREATE(event)

    def process_IN_MOVE_SELF(self, event):
        """
        File or directory is moved
        """
        # FIXME: image and music inotify also need to implement this method

    def process_IN_DELETE_SELF(self, event):
        """
        Listened directory was deleted
        """
        video_cache = VideoCache()
        if video_cache.isDirectoryInCache(
            os.path.join(event.path, event.name)):
            video_cache.removeDirectory(os.path.join(event.path, event.name))
            self.__fs_observer.removeFolder(
                os.path.join(event.path, event.name))

    def process_default(self, event):
        """
        Ultimately, this method is called for all others kind of events.
        """
        pass

