'''ArtistScreen - Screen allows user to browse music of one specific artist'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import pango
import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.transitions.transition import Transition
from entertainerlib.frontend.gui.widgets.image_menu import ImageMenu
from entertainerlib.frontend.gui.widgets.image_menu_item import ImageMenuItem
from entertainerlib.frontend.gui.widgets.justified_text_menu_item import (
    JustifiedTextMenuItem)
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.tabs_menu import TabsMenu
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.medialibrary.playlist import Playlist
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.configuration import Configuration

class ArtistScreen(Screen):
    """
    ArtistScreen

    Screen that allows user to browse music by artist.
    """

    def __init__(self, ui, music_library, artist):
        """Create the layout of the screen"""

        self.config = Configuration()
        Screen.__init__(self)

        self.ui = ui
        self.player = ui.get_media_player()
        self.theme = self.config.theme
        self.library = music_library
        self.artist = artist

        # Screen Title (Displayed at the bottom left corner)
        screen_title = Label(0.13, "screentitle", 0, 0.87, self.artist)
        screen_title.set_ellipsize(pango.ELLIPSIZE_END)
        screen_title.width = 1
        self.add(screen_title)

        # Create tabs and content
        self.album_artist = None
        self.album_title = None
        self.album_tracks = None
        self.album_menu = None
        self.album_li = None
        self.albums_tab = None
        self._create_albums_tab()

        self.track_title = None
        self.track_number = None
        self.track_length = None
        self.track_li = None
        self.tracks_menu = None
        self.tracks_tab = None
        self._create_tracks_tab()

        self.tabs = None
        self._create_tabs()

        # Activate albums tab when screen is opened
        self.current_tab = "albums"
        self.albums_tab.show()

        self.show()

    def _create_tabs(self):
        """
        Create tab bar of the screen. Tabs: Movies, TV-Series, Video clips
        """
        self.tabs = TabsMenu(self.theme, self.config.show_effects())
        self.tabs.set_item_size(self.ui.x(0.2196), self.ui.y(0.0781))
        self.tabs.set_visible_column_count(4)

        tab1 = JustifiedTextMenuItem(0.2196, _("Albums"))
        tab1.set_userdata("albums")
        self.tabs.add_actor(tab1)

        tab2 = JustifiedTextMenuItem(0.2196, _("Tracks"))
        tab2.set_userdata("tracks")
        self.tabs.add_actor(tab2)

        self.tabs.set_position(self.ui.x(0.2804), self.ui.y(0.05208))
        self.tabs.set_active(True)
        self.tabs.show()
        self.add(self.tabs)

    def _create_tracks_tab(self):
        """
        Create a list of tracks
        """
        self.tracks_tab = clutter.Group()
        self.tracks_menu = TextMenu(self.theme, self.config.show_effects())
        self.tracks_menu.set_item_size(self.ui.x(0.2928), self.ui.y(0.0781))
        cursor = clutter.Rectangle()
        cursor.set_color((0, 0, 0, 0))
        self.tracks_menu.set_cursor(cursor)

        tracks = self.library.get_tracks_by_artist(self.artist)
        for track in tracks:
            item = JustifiedTextMenuItem(0.2928, track.get_title())
            item.set_userdata(track)
            self.tracks_menu.add_actor(item)

        self.tracks_menu.set_orientation(TextMenu.HORIZONTAL)
        self.tracks_menu.set_row_count(7)
        self.tracks_menu.set_visible_column_count(3)

        self.tracks_menu.set_position(self.ui.x(0.0586), self.ui.y(0.2083))
        self.tracks_menu.set_active(False)
        self.tracks_menu.show()

        if self.config.show_effects():
            self.tracks_menu.set_animate(True)

        self.tracks_tab.add(self.tracks_menu)

        self.track_title = Label(0.045, "title", 0.22, 0.79, "")
        self.track_title.set_ellipsize(pango.ELLIPSIZE_END)
        self.track_title.width = 0.366
        self.tracks_tab.add(self.track_title)

        self.track_number = Label(0.037, "subtitle", 0.22, 0.86, "")
        self.track_number.set_ellipsize(pango.ELLIPSIZE_END)
        self.track_number.width = 0.366
        self.tracks_tab.add(self.track_number)

        self.track_length = Label(0.037, "subtitle", 0.22, 0.91, "")
        self.tracks_tab.add(self.track_length)

        # Create artis menu list indicator
        self.track_li = ListIndicator(0.18, 0.045, ListIndicator.HORIZONTAL)
        self.track_li.set_position(self.ui.x(0.9297) - \
            self.track_li.get_width(), self.ui.y(0.7943))
        self.track_li.set_delimiter(" | ")
        self.track_li.set_maximum(self.tracks_menu.get_number_of_items())
        self.track_li.set_current(1)
        self.track_li.show()
        self.tracks_tab.add(self.track_li)

        self.add(self.tracks_tab)

    def _create_albums_tab(self):
        """
        Create a list of albums
        """
        self.albums_tab = clutter.Group()
        albums = self.library.get_albums_by_artist(self.artist)

        if len(albums) < 4:
            # XXX: laymansterms - This is "less than elegant." I'm not fixing
            # ImageMenu right now so we can't get rid of tex_size_x and y yet.
            # We really need to eliminate the concept tex_size_y, but it will
            # have to wait for now.
            x_percent = 0.2928
            tex_size_x = self.ui.x(x_percent)
            tex_size_y = self.ui.y(0.5208)
            row_count = 1
            visible = 3
        elif len(albums) < 13:
            x_percent = 0.1464
            tex_size_x = self.ui.x(x_percent)
            tex_size_y = self.ui.y(0.2604)
            row_count = 2
            visible = 6
        else:
            x_percent = 0.1098
            tex_size_x = self.ui.x(x_percent)
            tex_size_y = self.ui.y(0.1953)
            row_count = 3
            visible = 8

        # Create albums menu
        self.album_menu = ImageMenu(self.theme)
        self.album_menu.set_row_count(row_count)
        self.album_menu.set_visible_column_count(visible)
        self.album_menu.set_item_size(tex_size_x, tex_size_y)

        # Default texture
        default_cover = Texture(self.theme.getImage("default_album_art"))

        for album in albums:
            if album.has_album_art():
                texture = Texture(album.get_album_art_url())
            else:
                texture = clutter.CloneTexture(default_cover)

            item = ImageMenuItem(x_percent, 1, texture)
            item.set_userdata(album)
            self.album_menu.add_actor(item)

        self.album_menu.set_position(self.ui.x(0.0586), self.ui.y(0.1823))
        self.album_menu.set_active(False)
        self.album_menu.show()

        if self.config.show_effects():
            self.album_menu.set_animate(True)

        self.albums_tab.add(self.album_menu)

        # Create album menu list indicator
        self.album_li = ListIndicator(0.18, 0.045, ListIndicator.HORIZONTAL)
        self.album_li.set_position(self.ui.x(0.9297) - \
            self.album_li.get_width(), self.ui.y(0.7943))
        self.album_li.set_delimiter(" | ")
        self.album_li.set_maximum(self.album_menu.get_number_of_items())
        self.album_li.set_current(1)
        self.album_li.show()
        self.albums_tab.add(self.album_li)

        # Create album information (displays current menuitem information)
        self.album_title = Label(0.045, "title", 0.22, 0.79, "")
        self.album_title.set_ellipsize(pango.ELLIPSIZE_END)
        self.album_title.width = 0.366
        self.albums_tab.add(self.album_title)

        self.album_artist = Label(0.037, "subtitle", 0.22, 0.86, "")
        self.album_artist.set_ellipsize(pango.ELLIPSIZE_END)
        self.album_artist.width = 0.366
        self.albums_tab.add(self.album_artist)

        self.album_tracks = Label(0.037, "subtitle", 0.22, 0.91, "")
        self.albums_tab.add(self.album_tracks)

        self.add(self.albums_tab)

    def _update_album_info(self, album):
        """
        Update album information. This information is displayed on albums tab.
        @param album: Album object
        """
        self.album_title.set_text(album.get_title())
        self.album_artist.set_text(album.get_artist())
        self.album_tracks.set_text(_("%(total)s tracks") % \
            {'total': album.get_number_of_tracks()})

    def _update_track_info(self, track):
        """
        Update track information. This information is displayed on track tab.
        @param track: Track object
        """
        self.track_title.set_text(track.get_title())
        self.track_length.set_text(str(track.get_length() / 60) + ":" + \
            str(track.get_length() % 60).zfill(2))
        self.track_number.set_text(_("Track %(track)s from %(album)s") % \
            {'track': track.get_tracknumber(),
             'album': track.get_album().get_title()})

    def _change_tab(self):
        """
        Change tab. This is called when user changes the screen tab.
        """
        self.tracks_tab.hide()
        self.albums_tab.hide()

        self.current_tab = self.tabs.get_current_menuitem().get_userdata()
        if self.current_tab == "tracks":
            self.tracks_tab.show()
        else:
            self.albums_tab.show()

    def is_interested_in_play_action(self):
        """
        Override function from Screen class. See Screen class for 
        better documentation.
        """
        if self.album_menu.is_active():
            return True
        else:
            return False

    def execute_play_action(self):
        """
        Override function from Screen class. See Screen class for 
        better documentation.
        """
        album = self.album_menu.get_current_menuitem().get_userdata()
        self.player.set_playlist(Playlist(album.get_tracks()))
        self.player.play()

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "ArtistScreen"

    def handle_user_event(self, event):   
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        if event_type == UserEvent.NAVIGATE_UP:
            if self.current_tab == "tracks":
                if self.tracks_menu.get_cursor_position()[1] == 0:
                    self.tracks_menu.set_active(False)
                    self.tabs.set_active(True)
                else:
                    self.tracks_menu.move(self.album_menu.UP)
                    self._update_track_info(
                        self.tracks_menu.get_current_menuitem().get_userdata())
                    self.track_li.set_current(
                        self.tracks_menu.get_current_postition() + 1)
            elif self.current_tab == "albums":
                if self.album_menu.get_cursor_position()[1] == 0:
                    self.album_menu.set_active(False)
                    self.tabs.set_active(True)
                else:
                    self.album_menu.move(self.album_menu.UP)
                    self._update_album_info(
                        self.album_menu.get_current_menuitem().get_userdata())
                    self.album_li.set_current(
                        self.album_menu.get_current_postition() + 1)
        elif event_type == UserEvent.NAVIGATE_DOWN:
            # Activate tab menu if tab bar was active
            if self.tabs.is_active():
                self.tabs.set_active(False)
                if self.current_tab == "tracks":
                    self.tracks_menu.set_active(True)
                    self._update_track_info(
                        self.tracks_menu.get_current_menuitem().get_userdata())
                elif self.current_tab == "albums":
                    self.album_menu.set_active(True)
                    self._update_album_info(
                        self.album_menu.get_current_menuitem().get_userdata())
                return
            if self.current_tab == "tracks":
                self.tracks_menu.move(self.tracks_menu.DOWN)
                self._update_track_info(
                    self.tracks_menu.get_current_menuitem().get_userdata())
                self.track_li.set_current(
                    self.tracks_menu.get_current_postition() + 1)
            elif self.current_tab == "albums":
                self.album_menu.move(self.album_menu.DOWN)
                self._update_album_info(
                    self.album_menu.get_current_menuitem().get_userdata())
                self.album_li.set_current(
                    self.album_menu.get_current_postition() + 1)
        elif event_type == UserEvent.NAVIGATE_LEFT:
            if self.tabs.is_active():
                self.tabs.move(self.tabs.LEFT)
                self._change_tab()
            elif self.current_tab == "tracks":
                self.tracks_menu.move(self.tracks_menu.LEFT)
                self._update_track_info(
                    self.tracks_menu.get_current_menuitem().get_userdata())
                self.track_li.set_current(
                    self.tracks_menu.get_current_postition() + 1)
            elif self.current_tab == "albums":
                self.album_menu.move(self.album_menu.LEFT)
                self._update_album_info(
                    self.album_menu.get_current_menuitem().get_userdata())
                self.album_li.set_current(
                    self.album_menu.get_current_postition() + 1)
        elif event_type == UserEvent.NAVIGATE_RIGHT:
            if self.tabs.is_active():
                self.tabs.move(self.tabs.RIGHT)
                self._change_tab()  
            elif self.current_tab == "tracks":
                self.tracks_menu.move(self.tracks_menu.RIGHT)
                self._update_track_info(
                    self.tracks_menu.get_current_menuitem().get_userdata())
                self.track_li.set_current(
                    self.tracks_menu.get_current_postition() + 1)
            elif self.current_tab == "albums":
                self.album_menu.move(self.album_menu.RIGHT)
                self._update_album_info(
                    self.album_menu.get_current_menuitem().get_userdata())
                self.album_li.set_current(
                    self.album_menu.get_current_postition() + 1)
        elif event_type == UserEvent.NAVIGATE_SELECT:
            if self.current_tab == "tracks":
                track = self.tracks_menu.get_current_menuitem().get_userdata()
                self.ui.changeScreen(self.ui.createScreen("audio_play", track),
                    Transition.FORWARD)
            elif self.current_tab == "albums":
                album = self.album_menu.get_current_menuitem().get_userdata()
                self.ui.changeScreen(self.ui.createScreen("music_album", album),
                    Transition.FORWARD)

