'''AudioPlayScreen - Screen displays information of currently playing audio'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import gtk
import pango
import gobject
import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.tabs.tab import Tab
from entertainerlib.frontend.gui.widgets.eyecandy_texture import (
    EyeCandyTexture)
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.loading_animation import (
    LoadingAnimation)
from entertainerlib.frontend.gui.widgets.progress_bar import ProgressBar
from entertainerlib.frontend.gui.widgets.scroll_area import ScrollArea
from entertainerlib.frontend.gui.widgets.tab_group import TabGroup
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.configuration import Configuration

class AudioPlayScreen(Screen):
    """
    AudioPlayScreen

    Screen that displays information of currently playing audio track.
    """

    def __init__(self, ui, music_library, track):
        """Create the layout of the screen"""

        self.config = Configuration()
        Screen.__init__(self)

        self.ui = ui
        self.player = ui.get_media_player()
        self.theme = self.config.theme
        self.library = music_library
        self.track = track
        self.album = track.get_album()

        # Create album art (this is displayed on all tab pages
        if(self.album.has_album_art()):
            pixbuf = gtk.gdk.pixbuf_new_from_file(
                self.album.get_album_art_url())
        else:
            pixbuf = gtk.gdk.pixbuf_new_from_file(
                self.theme.getImage("default_album_art"))
        self.art = EyeCandyTexture(pixbuf, self.ui.x(0.3148),
            self.ui.y(0.5599))
        self.art.set_position(self.ui.get_abs_x_pos(0.1),
            self.ui.get_abs_y_pos(0.22))
        self.art.set_rotation(clutter.Y_AXIS, 25, 0, 0, 0)
        self.art.show()
        self.add(self.art)

        # Tabs
        self.tab_group = TabGroup(0.95, 0.13, "title")
        self.progress_bar = None
        self.track_length = None
        self.current_pos = None
        self.artist_label = None
        self.album_label = None
        self.track_label = None
        tab1 = self._create_currently_playing_tab()
        tab2 = self._create_lyrics_tab()
        tab3 = self._create_playlist_tab()
        self.tab_group.add_tab(tab1)
        self.tab_group.add_tab(tab2)
        self.tab_group.add_tab(tab3)
        self.tab_group.set_y(self.ui.y(0.1))
        self.tab_group.set_anchor_point_from_gravity(clutter.GRAVITY_CENTER)
        self.tab_group.set_x(self.ui.x(0.5))
        self.tab_group.show()
        self.tab_group.set_active(True)
        self.add(self.tab_group)

        self.add(tab1)
        self.add(tab2)
        self.add(tab3)

        self.show()

        self.ui.get_media_player().set_media(self.track)
        self.ui.get_media_player().play()
        gobject.timeout_add(250, self._update_progress_bar) # 1/4 sec

    def _create_currently_playing_tab(self):
        """
        Create a tab that shows information on currenlty playing audio track.
        """
        currently_playing_tab = Tab(self.ui, "Currently playing")

        # Track name
        if self.track.get_tracknumber() == 0:
            track_label_text = str(self.track.get_title())
        else:
            track_label_text = str(self.track.get_tracknumber()) + \
                ". " + self.track.get_title()

        self.track_label = Label(0.05, "text", 0.5, 0.33, track_label_text)
        self.track_label.set_ellipsize(pango.ELLIPSIZE_END)
        self.track_label.width = 0.4
        currently_playing_tab.add(self.track_label)

        # Album name
        if self.album.get_year() != 0:
            album_label_text = _("From %(title)s, %(year)s") % \
            {'title': self.album.get_title(), 'year': self.album.get_year()}
        else:
            album_label_text = _("From ") + self.album.get_title()

        self.album_label = Label(0.042, "subtitle", 0.5, 0.43, album_label_text)
        self.album_label.set_ellipsize(pango.ELLIPSIZE_END)
        self.album_label.width = 0.4
        currently_playing_tab.add(self.album_label)

        # Artist name
        artist_text = _("By %(artist)s") % {'artist': self.track.get_artist()}
        self.artist_label = Label(0.042, "subtitle", 0.5, 0.53, artist_text)
        self.artist_label.set_ellipsize(pango.ELLIPSIZE_END)
        self.artist_label.width = 0.4
        currently_playing_tab.add(self.artist_label)

        # Current position as time stamp
        self.current_pos = Label(0.037, "title", 0.5, 0.665, "00:00",
            "current_pos")
        currently_playing_tab.add(self.current_pos)

        self.progress_bar = ProgressBar(self.ui.x(0.3001), self.ui.y(0.03906),
            self.theme.getColor("title"))
        self.progress_bar.set_name("progress_bar")
        self.progress_bar.set_position(self.ui.x(0.565), self.ui.y(0.667))
        self.progress_bar.show()
        currently_playing_tab.add(self.progress_bar)

        self.track_length = Label(0.037, "title", 0.871, 0.665,
            self.track.get_length_string())
        currently_playing_tab.add(self.track_length)

        return currently_playing_tab

    def _update_progress_bar(self):
        """
        Update progress bar
        """
        progress_bar = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "progress_bar")
        progress_bar.set_progress(self.player.get_media_position())
        current_pos = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "current_pos")
        current_pos.set_text(self.player.get_media_position_string())
        return True

    def _create_lyrics_tab(self):
        """
        Create a tab that displays track lyrics. This tab displays loading
        animation until lyrics are available. Lyrics are searched from the
        Internet if local lyrics doesn't exist.
        """
        lyrics_tab = Tab(self.ui, "Lyrics")

        if self.track.has_lyrics():
            lyrics = Label(0.037, "subtitle", 0, 0, self.track.get_lyrics())
            lyrics.set_line_wrap_mode(pango.WRAP_WORD)
            lyrics.width = 0.366

            lyrics_area = ScrollArea(0.4, 0.57, 0.5, 0.23, lyrics)
            lyrics_area.set_name("lyrics_area")
            lyrics_tab.add(lyrics_area)
        else:
            # Display throbber animation while searching for lyrics
            white = clutter.Color(255, 255, 255, 255)
            gray = clutter.Color(128, 128, 128, 128)
            throbber = LoadingAnimation(50, white, gray)
            throbber.set_name("throbber")
            throbber.set_position(self.ui.x(0.7), self.ui.y(0.5))
            throbber.show()
            lyrics_tab.add(throbber)
            self.track.fetch_lyrics(self._lyrics_search_callback)

        return lyrics_tab

    def _lyrics_search_callback(self, lyrics_text):
        """
        This function is called when lyrics search is over.
        @param lyrics: Lyrics or None if lyrics weren't found
        """
        #TODO: We could smoothly fade away throbber
        lyrics_tab = self.tab_group.get_tab_by_index(1)
        lyrics_tab.find_child_by_name("throbber").hide()

        if lyrics_text is None:
            no_lyrics = Label(0.037, "title", 0.5, 0.5,
                _("No lyrics found for this track"))
            lyrics_tab.add(no_lyrics)
        else:
            lyrics = Label(0.037, "subtitle", 0, 0, lyrics_text)
            lyrics.set_line_wrap_mode(pango.WRAP_WORD)
            lyrics.width = 0.366

            lyrics_area = ScrollArea(0.4, 0.57, 0.5, 0.23, lyrics)
            lyrics_area.set_name("lyrics_area")
            lyrics_tab.add(lyrics_area)
            self.library.save_lyrics(self.track, lyrics_text)

    def _create_playlist_tab(self):
        """
        Create a tab that displays tracks on current playlist.
        """
        playlist_tab = Tab(self.ui, _("Current playlist"))
        return playlist_tab

    def update(self, track):
        """
        This is called when track changes.
        @param track: Currently playing track
        """
        self.track = track
        self.album = track.get_album()

        if self.track.get_tracknumber() == 0:
            self.track_label.set_text(str(self.track.get_title()))
        else:
            self.track_label.set_text(str(self.track.get_tracknumber()) + \
                ". " + self.track.get_title())

        if self.album.get_year() != 0:
            self.album_label.set_text(_("From %(title)s, %(year)s") % \
            {'title': self.album.get_title(), 'year': self.album.get_year()})
        else:
            self.album_label.set_text(_("From %(album)s") % \
                {'album': self.album.get_title()})
            self.artist_label.set_text(_("By %(artist)s") % \
                {'artist': self.track.get_artist()})

        self.current_pos.set_text("00:00")
        self.track_length.set_text(self.track.get_length_string())
        self.progress_bar.set_progress(0)

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "AudioPlayScreen"

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        if event_type == UserEvent.NAVIGATE_UP:
            tab = self.tab_group.get_current_tab()

            # If we are on Lyrics tab
            #XXX: Josh - This is a horrible line, the title of the tab should
            #           not be being used here
            if tab.get_title() == _("Lyrics") and not \
                self.tab_group.is_active():
                area = tab.find_child_by_name("lyrics_area")
                if area.is_active() and area.get_offset() == 0:
                    area.set_active(False)
                    self.tab_group.set_active(True)
                elif area.is_active():
                    area.scroll_up()
        elif event_type == UserEvent.NAVIGATE_DOWN:
            tab = self.tab_group.get_current_tab()

            # If we are on Lyrics tab
            #XXX: Josh - This is a horrible line, the title of the tab should
            #           not be being used here
            if tab.get_title() == _("Lyrics"):
                area = tab.find_child_by_name("lyrics_area")
                if self.tab_group.is_active():
                    self.tab_group.set_active(False)
                    area = tab.find_child_by_name("lyrics_area")
                    try:
                        area.set_active(True)
                    except:
                        # There is no lyrics area since we are still searching
                        # for lyrics
                        pass
                else:
                    area.scroll_down()
        elif event_type == UserEvent.NAVIGATE_LEFT:
            if self.tab_group.is_active():
                self.tab_group.handle_user_event(event)
        elif event_type == UserEvent.NAVIGATE_RIGHT:
            if self.tab_group.is_active():
                self.tab_group.handle_user_event(event)
        elif event_type == UserEvent.NAVIGATE_SELECT:
            pass

