'''FeedEntryScreen - Screen displays one entry of some RSS feed'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import pango
import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.scroll_area import ScrollArea
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.feed_utils import FeedEntryParser
from entertainerlib.utils.configuration import Configuration

class FeedEntryScreen(Screen):
    """
    Feed entry Screen

    This screen displays one feed entry.
    """

    def __init__(self, ui, data):
        """Create the layout of the screen

        @param data: List [Feed object, Entry object]
        """
        self.config = Configuration()
        Screen.__init__(self)

        self.ui = ui
        self.theme = self.config.theme
        self.feed = data[0]
        self.entry = data[1]

        # Screen Title (Displayed at the bottom left corner)
        screen_title_text = FeedEntryParser().convert(self.feed.get_title())
        screen_title = Label(0.13, "screentitle", 0, 0.87, screen_title_text)
        screen_title.set_ellipsize(pango.ELLIPSIZE_END)
        screen_title.width = 0.8
        screen_title.set_use_markup(True)
        self.add(screen_title)

        # RSS Icon
        icon = Texture(self.theme.getImage("rss_icon"), 0.05, 0.07)
        self.add(icon)

        # Entry Title
        title_text = FeedEntryParser().convert(self.feed.get_title())
        title = Label(0.065, "title", 0.13, 0.07, title_text)
        title.set_use_markup(True)
        title.width = 0.6
        title.set_ellipsize(pango.ELLIPSIZE_END)
        self.add(title)

        # RSS Feed descrption
        full_text = self.entry.get_title()
        try:
            sub_title = full_text[:full_text.index('\n')]
        except ValueError:
            sub_title = full_text
        desc_text = FeedEntryParser().convert(sub_title)
        desc = Label(0.0365, "subtitle", 0.13, 0.14, desc_text)
        desc.set_use_markup(True)
        desc.set_ellipsize(pango.ELLIPSIZE_END)
        desc.width = 0.6
        self.add(desc)

        # Last update
        update = Label(0.0313, "title", 0.78, 0.1, _("Updated"))
        self.add(update)
        date_text = self.entry.get_time()[:5]  + " - " +  self.entry.get_date()
        date = Label(0.0313, "subtitle", 0.78, 0.14, date_text)
        self.add(date)

        # Entry descrption
        if len(self.entry.get_description().strip()) == 0:
            entry_text = Label(0.04167, "text", 0.35, 0.48,
                _("Entry has no content"))
            self.add(entry_text)
        else:
            entry_text_data = FeedEntryParser().convert(
                self.entry.get_description())
            entry_text = Label(0.0365, "text", 0, 0, entry_text_data)
            entry_text.set_use_markup(True)
            entry_text.set_size(0.5857, 0.5208)

            self.scroll_area = ScrollArea(0.5857, 0.4948, 0.2072, 0.3,
                entry_text, active=True)
            self.add(self.scroll_area)

        self.show()

        #We've got to the end without any problems so lets set the entry as read
        self.entry.mark_as_read()

    def get_type(self):
        """
        Get screen type.
        @return: Integer
        """
        return Screen.NORMAL

    def get_name(self):
        """
        Get screen name
        @return: String
        """
        return "EntryFeedScreen"

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        if event_type == UserEvent.NAVIGATE_UP:
            self.scroll_area.scroll_up()
        elif event_type == UserEvent.NAVIGATE_DOWN:
            self.scroll_area.scroll_down()
        elif event_type == UserEvent.NAVIGATE_FIRST_PAGE:
            self.scroll_area.scroll_to_top()
        elif event_type == UserEvent.NAVIGATE_PREVIOUS_PAGE:
            self.scroll_area.scroll_to_bottom()
        elif event_type == UserEvent.NAVIGATE_PREVIOUS_PAGE:
            self.scroll_area.scroll_page_up()
        elif event_type == UserEvent.NAVIGATE_NEXT_PAGE:
            self.scroll_area.scroll_page_down()

