'''MainScreen - Screen has a main menu'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.transitions.transition import Transition
from entertainerlib.frontend.gui.widgets.clock_label import ClockLabel
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.scroll_menu import ScrollMenu
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.cd_utils import eject_cd
from entertainerlib.utils.configuration import Configuration
from entertainerlib.utils.feed_utils import FeedEntryParser

class MainScreen(Screen):
    """
    MainScreen

    This screen is displayed when frontend is opened. Screen includes menu
    that allows user to navigate media center. This is main screen of the
    frontend.
    """

    # Size of the preview area
    PREVIEW_WIDTH = 830
    PREVIEW_HEIGHT = 580

    def __init__(self, ui, feed_library):
        """Create the layout of the screen"""

        self.config = Configuration()
        Screen.__init__(self)

        self.ui = ui
        self.theme = self.config.theme
        self.feed_library = feed_library

        self.preview = clutter.Group() # Group that contains preview actors
        self.preview.set_position(self.ui.x(0.07), self.ui.y(0.1))
        self.preview.show()
        self.add(self.preview)

        self.active_menu = "main"
        self.preview_menu = None # Pointer to the current preview menu

        self.menu = None
        self.playing_item = None
        self.create_main_menu()
        self.create_clock()

    def create_clock(self):
        """Create a clock label"""
        clock = ClockLabel()
        clock.set_font_name(self.theme.getFont() + " " + \
            str(self.ui.y(0.13)) + "px")
        clock.set_color(self.theme.getColor("screentitle"))
        clock.set_name("bgtitle")
        clock.set_position(self.ui.x(0), self.ui.y(0.87))
        clock.show()
        self.add(clock)

    def create_main_menu(self):
        """Create main menu of the home screen"""
        self.menu = ScrollMenu(10, 60)
        self.menu.set_name("mainmenu")

        # Common values for all ScrollMenu items
        size = 0.045
        color = "menuitem_active"

        item0 = Label(size, color, 0, 0, _("Play CD"), "disc")
        self.menu.add(item0)

        item1 = Label(size, color, 0, 0, _("Television"), "tv")
        self.menu.add(item1)

        item2 = Label(size, color, 0, 0, _("Videos"), "videos")
        self.menu.add(item2)

        item3 = Label(size, color, 0, 0, _("Music"), "music")
        self.menu.add(item3)

        item4 = Label(size, color, 0, 0, _("Photographs"), "photo")
        self.menu.add(item4)

        item5 = Label(size, color, 0, 0, _("Headlines"), "rss")
        self.menu.add(item5)

        if self.config.display_weather_in_frontend():
            item6 = Label(size, color, 0, 0, _("Weather"), "weather")
            self.menu.add(item6)

        if self.config.display_cd_eject_in_frontend():
            item7 = Label(size, color, 0, 0, _("Eject CD"), "eject_cd")
            self.menu.add(item7)

        if self.ui.get_media_player().has_media():
            playing = Label(size, color, 0, 0, _("Playing now..."), "playing")
            self.menu.add(playing)

        separator = Label(size, color, 0, 0)
        self.menu.add(separator)

        self.menu.set_number_of_visible_items(5)
        menu_clip = self.menu.get_number_of_visible_items() * 70

        # Menu position
        menu_y = int((self.ui.STAGE_HEIGHT - menu_clip + 10) / 2)
        self.menu.set_position(self.ui.x(0.75), menu_y)
        self.menu.show()

        self.add(self.menu)

    def show_playing_preview(self):
        """Create a group that displays information on current media."""
        player = self.ui.get_media_player()

        # Video preview of current media
        video_texture = player.get_texture()
        width, height = video_texture.get_size()
        x_ratio = (self.PREVIEW_WIDTH - 50) / float(width)
        y_ratio = (self.PREVIEW_HEIGHT - 50) / float(height)

        if x_ratio > y_ratio:
            video_texture.set_scale((self.PREVIEW_HEIGHT - 50) / float(height),
                                    (self.PREVIEW_HEIGHT - 50) / float(height))
            new_width = int(width * \
                ((self.PREVIEW_HEIGHT - 50) / float(height)))
            new_x = int(((self.PREVIEW_WIDTH - 50) - new_width) / 2.0)
            video_texture.set_position(int(new_x), 0)
            # Below are size and position calculations for border rectangle
            rect_x = new_x -3
            rect_y = -3
            new_width = (self.PREVIEW_HEIGHT - 50) / float(height) * width
            new_height = (self.PREVIEW_HEIGHT - 50) / float(height) * height
        else:
            video_texture.set_scale((self.PREVIEW_WIDTH - 50) / float(width),
                                    (self.PREVIEW_WIDTH - 50) / float(width))
            new_height = int(height * \
                ((self.PREVIEW_WIDTH - 50) / float(width)))
            new_y = int(((self.PREVIEW_HEIGHT - 50) - new_height) / 2.0)
            video_texture.set_position(0, int(new_y))
            rect_x = -3
            rect_y = new_y -3
            # Below are size and position calculations for border rectangle
            new_width = (self.PREVIEW_WIDTH - 50) / float(width) * width
            new_height = (self.PREVIEW_WIDTH - 50) / float(width) * height
        video_texture.show()

        # Video frame
        rect = clutter.Rectangle()
        rect.set_size(int(new_width + 6), int(new_height + 6))
        rect.set_position(rect_x, rect_y)
        rect.set_color((128, 128, 128, 192))
        rect.show()
        self.preview.add(rect)

        self.preview.add(video_texture)

        title_text = _("Now Playing: %(title)s") % \
            {'title': player.get_media_title()}
        title = Label(0.03, "text", 0.03, 0.74, title_text)
        self.preview.add(title)

    def show_rss_preview(self):
        """Rss preview"""

        if self.feed_library.is_empty() is False:
            # RSS Icon
            icon = Texture(self.theme.getImage("rss_icon"), 0, 0.04)
            icon.set_scale(0.6, 0.6)

            # RSS Feed Title
            title = Label(0.075, "title", 0.045, 0.03, _("Recent headlines"))

            menu = TextMenu(self.theme, self.config.show_effects())
            menu.set_row_count(1)
            menu.set_position(self.ui.x(0.035), self.ui.y(0.12))
            menu.set_item_size(self.ui.x(0.549), self.ui.y(0.078))

            # List of latest entries. pack = (Feed object, Entry object)
            entries = self.feed_library.get_latest_entries(5)
            for pack in entries:
                text = pack[0].get_title() + " - " + pack[1].get_title()
                item = TextMenuItem(0.549, 0.078,
                    FeedEntryParser().strip_tags(text),
                    pack[1].get_date())
                item.set_userdata([pack[0], pack[1]])
                menu.add_actor(item)

            menu.set_active(False)

            # Fade in timeline
            fade_in = clutter.Timeline(20, 60)
            alpha_in = clutter.Alpha(fade_in, clutter.smoothstep_inc_func)
            self.in_behaviour = clutter.BehaviourOpacity(0x00, 0xff, alpha_in)
            self.in_behaviour.apply(menu)
            self.in_behaviour.apply(title)
            self.in_behaviour.apply(icon)

            self.preview.add(icon)
            self.preview.add(title)
            self.preview_menu = menu
            self.preview.add(menu)

            menu.set_opacity(0x00)
            icon.set_opacity(0x00)
            title.set_opacity(0x00)

            menu.show()
            icon.show()
            title.show()

            fade_in.start()

        else:
            # XXX: laymansterms - I don't think this will ever happen. The
            # FeedManager creates an empty feed db if one doesn't exist. And
            # the is_empty() command just checks for the existence of a feed db
            # Since the FeedManager will likely always execute before this code,
            # this else statement won't execute. is_empty should probably check
            # if there is anything in the feed db, not just that the file exists
            # Since the RSS screen would tell users that there are no headlines,
            # it probably makes more sense to not show any message.

            # No headlines available in the library
            info = Label(0.07, "title", 0.1, 0.35, _("No headlines available"))
            self.preview.add(info)

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "MainScreen"

    def update(self):
        """
        Update screen widgets. This is called always when screen is poped from
        the screen history. Updates main menu widget.
        """
        selected_name = self.menu.get_selected().get_name()

        self.remove(self.menu)
        self.create_main_menu()

        index = self.menu.index(selected_name)
        if not index == -1:
            self.menu.select(index)

        self.update_preview_area(self.menu.get_selected())

    def update_preview_area(self, item):
        """
        Update preview area. This area displayes information of currently
        selected menuitem.
        @param item: Menuitem (clutter.Actor)
        """
        self.preview.remove_all()
        if item.get_name() == "playing":
            self.show_playing_preview()
        #only show the rss menu if it has entries
        elif item.get_name() == "rss" and not self.feed_library.is_empty():
            self.show_rss_preview()

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        # Scroll main menu
        if event_type == UserEvent.NAVIGATE_UP:
            if self.active_menu == "main":
                self.menu.scroll_down()
                selected = self.menu.get_selected()
                if selected.get_name() == None:
                    self.menu.scroll_down()
                    selected = self.menu.get_selected()
                self.update_preview_area(selected)
            else:
                self.preview_menu.move(TextMenu.UP)

        elif event_type == UserEvent.NAVIGATE_DOWN:
            if self.active_menu == "main":
                self.menu.scroll_up()
                selected = self.menu.get_selected()
                if selected.get_name() == None:
                    self.menu.scroll_up()
                    selected = self.menu.get_selected()
                self.update_preview_area(selected)
            else:
                self.preview_menu.move(TextMenu.DOWN)

        # Make preview menu active
        elif event_type == UserEvent.NAVIGATE_LEFT:
            #We should only show the preview menu if it has stuff in it
            if self.active_menu == "main" and \
               self.preview.get_n_children() > 0:
                self.active_menu = "preview"
                self.menu.set_active(False)
                self.preview_menu.set_active(True)
                #FIXME: REMOVE THIS CODE AND MOVE SELECTOR TO SCROLL MENU
                self.find_child_by_id(self.selector_id).hide()

        # Make feed menu active
        elif event_type == UserEvent.NAVIGATE_RIGHT:
            if self.active_menu == "preview":
                self.active_menu = "main"
                self.menu.set_active(True)
                self.preview_menu.set_active(False)
                #FIXME: REMOVE THIS CODE AND MOVE SELECTOR TO SCROLL MENU
                self.find_child_by_id(self.selector_id).show()

        # Menuitem selected
        elif event_type == UserEvent.NAVIGATE_SELECT:
            item = self.menu.get_selected()

            if item.get_name() == "disc":
                self.ui.changeScreen(self.ui.createScreen("audio_cd"),
                    Transition.FORWARD)
            elif item.get_name() == "playing":
                self.ui.changeScreen(self.ui.createScreen("video_osd"),
                    Transition.FORWARD)
            elif item.get_name() == "music":
                self.ui.changeScreen(self.ui.createScreen("music"),
                    Transition.FORWARD)
            elif item.get_name() == "tv":
                self.ui.changeScreen(self.ui.createScreen("tv_osd"),
                    Transition.FORWARD)
            elif item.get_name() == "videos":
                self.ui.changeScreen(self.ui.createScreen("videos"),
                    Transition.FORWARD)
            elif item.get_name() == "weather":
                self.ui.changeScreen(self.ui.createScreen("weather"),
                    Transition.FORWARD)
            elif item.get_name() == "eject_cd":
                eject_cd()
            elif item.get_name() == "photo":
                self.ui.changeScreen(self.ui.createScreen("photo_albums"),
                    Transition.FORWARD)
            elif item.get_name() == "rss":
                if self.active_menu == "main":
                    self.ui.changeScreen(self.ui.createScreen("rss"),
                        Transition.FORWARD)
                else:
                    menu_item = self.preview_menu.get_current_menuitem()
                    data = menu_item.get_userdata()
                    self.ui.changeScreen(self.ui.createScreen("entry", data),
                        Transition.FORWARD)


