'''MusicScreen - Screen allows user to select music he/she wants to listen'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import pango
import gobject
import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.tabs.tab import Tab
from entertainerlib.frontend.gui.transitions.transition import Transition
from entertainerlib.frontend.gui.widgets.image_menu import ImageMenu
from entertainerlib.frontend.gui.widgets.image_menu_item import ImageMenuItem
from entertainerlib.frontend.gui.widgets.justified_text_menu_item import (
    JustifiedTextMenuItem)
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.loading_animation import (
    LoadingAnimation)
from entertainerlib.frontend.gui.widgets.tab_group import TabGroup
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem
from entertainerlib.frontend.medialibrary.playlist import Playlist
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.configuration import Configuration
from entertainerlib.utils.logger import Logger

class MusicScreen(Screen):
    """
    MusicScreen

    Screen that allows user to browse music library content.
    """

    def __init__(self, ui, music_library):
        """Create the layout of the screen"""

        self.config = Configuration()
        Screen.__init__(self)

        self.logger = Logger().getLogger(
            'frontend.gui.screen.MusicScreen')
        self.ui = ui
        self.player = ui.get_media_player()
        self.theme = self.config.theme
        self.library = music_library

        # Screen Title (Displayed at the bottom left corner)
        screen_title = Label(0.13, "screentitle", 0, 0.87, _("Music"))
        self.add(screen_title)

        if self.library.number_of_tracks() == 0:
            self._create_no_music_information()
        else:
            # Tabs
            self.tab_group = TabGroup(0.95, 0.13, "title")
            self.behaviour = None
            tab1 = self._create_artists_tab()
            self.default_cover = None
            self.behaviour_album_throbber = None
            tab2 = self._create_albums_tab()
            self.playlist_menu = None
            tab3 = self._create_playlists_tab()
            self.tab_group.add_tab(tab1)
            self.tab_group.add_tab(tab2)
            self.tab_group.add_tab(tab3)
            self.tab_group.set_y(self.ui.y(0.1))
            self.tab_group.set_anchor_point_from_gravity(clutter.GRAVITY_CENTER)
            self.tab_group.set_x(self.ui.x(0.5))
            self.tab_group.show()
            self.tab_group.set_active(True)
            self.add(self.tab_group)

            self.add(tab1)
            self.add(tab2)
            self.add(tab3)

        self.show()

    def _async_artist_menu_build(self, artists):
        """
        This function is called as async callback. It is also recursive
        function. This allows us load menu items in the background without
        blocking the user interface.
        """
        artist_menu = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "artist_menu")
        artist_throbber = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "artist_throbber")

        # Check if we should stop recursive calls (menu is complete)
        if len(artists) > 0:
            artist = artists[0]
            item = JustifiedTextMenuItem(0.293, artist)
            item.set_userdata(artist)
            artist_menu.add_actor(item)
            # Recursive call, remove first artist from the list
            gobject.timeout_add(15, self._async_artist_menu_build, artists[1:])
            return False
        else:
            if self.config.show_effects():
                timeline = clutter.Timeline(35, 26)
                alpha = clutter.Alpha(timeline, clutter.smoothstep_inc_func)
                self.behaviour = clutter.BehaviourOpacity(255, 0, alpha)
                self.behaviour.apply(artist_throbber)
                timeline.start()
            else:
                artist_throbber.hide()
            return False

    def _create_no_music_information(self):
        """
        Create textures and labels for information screen. This is displayed
        instead of artist list if there are no tracks available and it helps
        users to add new music to the system.
        """
        # Create warning icon
        warning_icon = Texture(self.theme.getImage("warning_icon"), 0.28, 0.27)
        self.add(warning_icon)

        # Create warning title
        info_title = Label(0.0625, "title", 0.3367, 0.2709,
            _("No music available!"))
        self.add(info_title)

        # Create warning help text
        message = _(
            "There are no indexed artists in the Entertainer media "
            "library. To add music, start the Content management tool "
            "and open the 'Music' tab. Now click on the 'Add' button and "
            "select some folders which contain image files.")
        info = Label(0.0417, "menuitem_inactive", 0.2804, 0.45, message)
        info.width = 0.5
        self.add(info)

    def _create_artists_tab(self):
        """
        Create a list of artists
        """
        artists_tab = Tab(self.ui, _("Artists"))
        artists_tab.set_name("artists_tab")

        artist_menu = TextMenu(self.theme, self.config.show_effects())
        artist_menu.set_name("artist_menu")
        artist_menu.set_item_size(self.ui.x(0.293), self.ui.y(0.078))
        cursor = clutter.Rectangle()
        cursor.set_color((0, 0, 0, 0))
        artist_menu.set_cursor(cursor)
        artist_menu.set_orientation(TextMenu.HORIZONTAL)
        artist_menu.set_row_count(7)
        artist_menu.set_visible_column_count(3)
        artist_menu.set_position(self.ui.x(0.057), self.ui.y(0.208))
        artist_menu.set_active(False)
        artist_menu.show()
        if self.config.show_effects():
            artist_menu.set_animate(True)
        artists_tab.add(artist_menu)

        # Display throbber animation while loading artist list
        artist_throbber = LoadingAnimation(40,
            clutter.Color(255, 255, 255, 255),
            clutter.Color(128, 128, 128, 128))
        artist_throbber.set_name("artist_throbber")
        artist_throbber.set_position(self.ui.x(0.25), self.ui.y(0.1))
        artist_throbber.show()
        artists_tab.add(artist_throbber)

        # Create artist label
        artist_title = Label(0.0416, "title", 0.22, 0.794, " ", "artist_title")
        artist_title.set_ellipsize(pango.ELLIPSIZE_END)
        artist_title.set_size(0.366, 0.052)
        artists_tab.add(artist_title)

        artist_albums = Label(0.0365, "subtitle", 0.22, 0.86, " ",
            "artist_albums")
        artists_tab.add(artist_albums)

        artist_tracks = Label(0.0365, "subtitle", 0.22, 0.911, " ",
            "artist_tracks")
        artists_tab.add(artist_tracks)

        # Create artis menu list indicator
        artist_li = ListIndicator(0.18, 0.045, ListIndicator.HORIZONTAL)
        artist_li.set_name("list_indicator")
        artist_li.set_position(self.ui.x(0.929) - artist_li.get_width(),
            self.ui.y(0.794))
        artist_li.set_delimiter(" | ")
        artist_li.set_maximum(len(self.library.get_all_artists()))
        artist_li.set_current(1)
        artist_li.show()
        artists_tab.add(artist_li)

        gobject.timeout_add(25, self._async_artist_menu_build,
            self.library.get_all_artists())

        return artists_tab

    def _async_album_menu_build(self, albums):
        """
        This function is called as async callback. It is also recursive
        function. This allows us load menu items in the background without
        blocking the user interface.
        """
        album_menu = self.tab_group.get_tab_by_index(1).find_child_by_name(
            "album_menu")
        album_throbber = self.tab_group.get_tab_by_index(1).find_child_by_name(
            "album_throbber")

        # Check if we should stop recursive calls (menu is complete)
        if len(albums) > 0:
            album = albums[0]
            if album.has_album_art():
                texture = Texture(album.get_album_art_url())
            else:
                texture = clutter.CloneTexture(self.default_cover)

            item = ImageMenuItem(0.11, 1, texture)
            item.set_userdata(album)
            album_menu.add_actor(item)
            # Recursive call, remove first album from the list
            gobject.timeout_add(25, self._async_album_menu_build, albums[1:])
            return False
        else:
            if self.config.show_effects():
                timeline_album_throbber = clutter.Timeline(35, 26)
                alpha_album_throbber = clutter.Alpha(
                    timeline_album_throbber, clutter.smoothstep_inc_func)
                self.behaviour_album_throbber = clutter.BehaviourOpacity(
                    255, 0, alpha_album_throbber)
                self.behaviour_album_throbber.apply(album_throbber)
                timeline_album_throbber.start()
            else:
                album_throbber.hide()
            return False

    def _create_albums_tab(self):
        """
        Create a list of albums
        """
        albums_tab = Tab(self.ui, _("Albums"))
        albums_tab.set_name("albums_tab")

        # Create albums menu
        album_menu = ImageMenu(self.theme)
        album_menu.set_name("album_menu")
        album_menu.set_item_size(self.ui.x(0.1098), self.ui.y(0.1953125))
        album_menu.set_position(self.ui.x(0.0586), self.ui.y(0.1822))
        album_menu.set_active(False)
        album_menu.show()

        # Load default albumart texture
        self.default_cover = Texture(self.theme.getImage("default_album_art"))

        if self.config.show_effects():
            album_menu.set_animate(True)

        albums_tab.add(album_menu)

        # Display throbber animation while loading artist list
        album_throbber = LoadingAnimation(40, clutter.Color(255, 255, 255, 255),
            clutter.Color(128, 128, 128, 128))
        album_throbber.set_name("album_throbber")
        album_throbber.set_position(self.ui.x(0.57), self.ui.y(0.1))
        album_throbber.show()
        albums_tab.add(album_throbber)

        # Create album menu list indicator
        album_li = ListIndicator(0.18, 0.045, ListIndicator.HORIZONTAL)
        album_li.set_name("list_indicator")
        album_li.set_position(self.ui.x(0.9297) - album_li.get_width(),
            self.ui.y(0.794))
        album_li.set_delimiter(" | ")
        album_li.set_maximum(len(self.library.get_all_albums()))
        album_li.set_current(1)
        album_li.show()
        albums_tab.add(album_li)

        # Begin menu load in the background
        gobject.timeout_add(50, self._async_album_menu_build,
            self.library.get_all_albums())

        # Create album information (displays current menu item information)
        album_title = Label(0.0416, "title", 0.219, 0.794, " ", "album_title")
        album_title.set_ellipsize(pango.ELLIPSIZE_END)
        album_title.set_size(0.366, 0.0521)
        albums_tab.add(album_title)

        album_artist = Label(0.0365, "subtitle", 0.219, 0.859375, " ",
            "album_artist")
        album_artist.set_ellipsize(pango.ELLIPSIZE_END)
        album_artist.set_size(0.366, 0.0521)
        albums_tab.add(album_artist)

        album_tracks = Label(0.0365, "subtitle", 0.219, 0.9115, " ",
            "album_tracks")
        albums_tab.add(album_tracks)

        return albums_tab

    def _create_playlists_tab(self):
        """
        Create a list of playlists
        """
        playlists_tab = Tab(self.ui, _("Playlists"))
        playlists_tab.set_name("playlists_tab")

        playlists = self.library.get_playlists()

        if len(playlists) == 0:
            empty_label = Label(0.05, "title", 0.5, 0.5,
                _("No saved playlists available."))
            empty_label.set_anchor_point_from_gravity(clutter.GRAVITY_CENTER)
            playlists_tab.add(empty_label)
        else:
            self.playlist_menu = TextMenu(self.theme,
                self.config.show_effects())
            self.playlist_menu.set_name("playlist_menu")
            self.playlist_menu.set_item_size(self.ui.x(0.9115),
                self.ui.y(0.07813))

            for playlist in playlists:
                item = TextMenuItem(0.9115, 0.07813, playlist)
                item.set_userdata(playlist)
                self.playlist_menu.add_actor(item)

            self.playlist_menu.set_position(self.ui.x(0.4026),
                self.ui.y(0.1823))
            self.playlist_menu.show()

            if self.config.show_effects():
                self.playlist_menu.set_animate(True)

            playlists_tab.add(self.playlist_menu)

        return playlists_tab

    def _update_album_info(self, album):
        """
        Update album information. This information is displayed on albums tab.
        @param album: Album object
        """
        album_title = self.tab_group.get_tab_by_index(1).find_child_by_name(
            "album_title")
        album_title.set_text(album.get_title())
        album_artist = self.tab_group.get_tab_by_index(1).find_child_by_name(
            "album_artist")
        album_artist.set_text(album.get_artist())
        album_tracks = self.tab_group.get_tab_by_index(1).find_child_by_name(
            "album_tracks")
        album_tracks.set_text(_("%(tracks)d tracks") %
            {'tracks': album.get_number_of_tracks()})

    def _update_artist_info(self, artist):
        """
        Update artist information. This information is displayed on artist tab.
        @param artist: Artist name
        """
        artist_title = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "artist_title")
        artist_title.set_text(artist)
        artist_albums = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "artist_albums")
        artist_albums.set_text(_("%(albums)d albums") %
            {'album': self.library.number_of_albums_by_artist(artist)})
        artist_tracks = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "artist_tracks")
        artist_tracks.set_text(_("%(tracks)d tracks") %
            {'tracks': self.library.number_of_tracks_by_artist(artist)})

    def get_type(self):
        """
        Get screen type.
        @return: Integer. Constant defined in Screen class
        """
        return Screen.NORMAL

    def get_name(self):
        """
        Get screen name (human readable)
        @return: String
        """
        return "MusicScreen"

    def is_interested_in_play_action(self):
        """
        Override function from Screen class. See Screen class for
        better documentation.
        @return: Boolean
        """
        artist_menu = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "artist_menu")
        album_menu = self.tab_group.get_tab_by_index(1).find_child_by_name(
            "album_menu")

        if artist_menu.is_active() or album_menu.is_active():
            return True
        else:
            return False

    def execute_play_action(self):
        """
        Override function from Screen class. See Screen class for
        better documentation.
        """
        artist_menu = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "artist_menu")
        album_menu = self.tab_group.get_tab_by_index(1).find_child_by_name(
            "album_menu")

        if artist_menu.is_active():
            artist = artist_menu.get_current_menuitem().get_userdata()
            self.player.set_playlist(Playlist(
                self.library.get_tracks_by_artist(artist)))
            self.player.play()
        elif album_menu.is_active():
            album = album_menu.get_current_menuitem().get_userdata()
            self.player.set_playlist(Playlist(album.get_tracks()))
            self.player.play()

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        # Don't handle input if there is nothing in the library
        if self.library.number_of_tracks() == 0:
            return

        event_type = event.get_type()

        tab = self.tab_group.get_current_tab()
        artist_menu = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "artist_menu")
        artist_li = self.tab_group.get_tab_by_index(0).find_child_by_name(
            "list_indicator")
        album_menu = self.tab_group.get_tab_by_index(1).find_child_by_name(
            "album_menu")
        album_li = self.tab_group.get_tab_by_index(1).find_child_by_name(
            "list_indicator")

        if event_type == UserEvent.NAVIGATE_UP:
            if not self.tab_group.is_active():
                if tab.get_name() == "artists_tab":
                    if artist_menu.get_cursor_position()[1] == 0:
                        artist_menu.set_active(False)
                        self.tab_group.set_active(True)
                    else:
                        artist_menu.move(artist_menu.UP)
                        self._update_artist_info(
                            artist_menu.get_current_menuitem().get_userdata())
                        artist_li.set_current(
                            artist_menu.get_current_postition() + 1)
                elif tab.get_name() == "albums_tab":
                    if album_menu.get_cursor_position()[1] == 0:
                        album_menu.set_active(False)
                        self.tab_group.set_active(True)
                    else:
                        album_menu.move(album_menu.UP)
                        self._update_album_info(
                            album_menu.get_current_menuitem().get_userdata())
                        album_li.set_current(
                            album_menu.get_current_postition() + 1)
                elif tab.get_name() == "playlists_tab":
                    pass

        elif event_type == UserEvent.NAVIGATE_DOWN:
            if self.tab_group.is_active():
                if tab.get_name() == "artists_tab":
                    artist_menu.set_active(True)
                    self.tab_group.set_active(False)
                    self._update_artist_info(
                        artist_menu.get_current_menuitem().get_userdata())
                elif tab.get_name() == "albums_tab":
                    album_menu.set_active(True)
                    self.tab_group.set_active(False)
                    self._update_album_info(
                        album_menu.get_current_menuitem().get_userdata())
                elif tab.get_name() == "playlists_tab":
                    try:
                        self.playlist_menu.set_active(True)
                        self.tab_group.set_active(False)
                    except:
                        self.logger.error(
                        'Could not set playlist menu or tab group to active')
                return
            else:
                if tab.get_name() == "artists_tab":
                    artist_menu.move(artist_menu.DOWN)
                    self._update_artist_info(
                        artist_menu.get_current_menuitem().get_userdata())
                    artist_li.set_current(
                        artist_menu.get_current_postition() + 1)
                elif tab.get_name() == "albums_tab":
                    album_menu.move(album_menu.DOWN)
                    self._update_album_info(
                        album_menu.get_current_menuitem().get_userdata())
                    album_li.set_current(album_menu.get_current_postition() + 1)
                elif tab.get_name() == "playlists_tab":
                    pass

        elif event_type == UserEvent.NAVIGATE_LEFT:
            if self.tab_group.is_active():
                self.tab_group.handle_user_event(event)
            else:
                if tab.get_name() == "artists_tab":
                    artist_menu.move(artist_menu.LEFT)
                    self._update_artist_info(
                        artist_menu.get_current_menuitem().get_userdata())
                    artist_li.set_current(
                        artist_menu.get_current_postition() + 1)
                elif tab.get_name() == "albums_tab":
                    album_menu.move(album_menu.LEFT)
                    self._update_album_info(
                        album_menu.get_current_menuitem().get_userdata())
                    album_li.set_current(album_menu.get_current_postition() + 1)
                elif tab.get_name() == "playlists_tab":
                    pass

        elif event_type == UserEvent.NAVIGATE_RIGHT:
            if self.tab_group.is_active():
                self.tab_group.handle_user_event(event)
            else:
                if tab.get_name() == "artists_tab":
                    artist_menu.move(artist_menu.RIGHT)
                    self._update_artist_info(
                        artist_menu.get_current_menuitem().get_userdata())
                    artist_li.set_current(
                        artist_menu.get_current_postition() + 1)
                elif tab.get_name() == "albums_tab":
                    album_menu.move(album_menu.RIGHT)
                    self._update_album_info(
                        album_menu.get_current_menuitem().get_userdata())
                    album_li.set_current(album_menu.get_current_postition() + 1)
                elif tab.get_name() == "playlists_tab":
                    pass

        elif event_type == UserEvent.NAVIGATE_SELECT:
            if not self.tab_group.is_active():
                if tab.get_name() == "artists_tab":
                    artist = artist_menu.get_current_menuitem().get_userdata()
                    self.ui.changeScreen(self.ui.createScreen("artist", artist),
                        Transition.FORWARD)
                elif tab.get_name() == "albums_tab":
                    album = album_menu.get_current_menuitem().get_userdata()
                    self.ui.changeScreen(self.ui.createScreen("music_album",
                        album), Transition.FORWARD)
                elif tab.get_name() == "playlists_tab":
                    pass

