'''PhotoScreen - Screen displays photograph in fullscreen'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import gobject
import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.configuration import Configuration

class PhotoScreen(Screen):
    """
    PhotoScreen

    This screen displays photograph in fullscreen. Screen also allows user to 
    zoom in and view photograph information.
    """

    # Constants for view
    PHOTO_VIEW = 0
    INFORMATION_VIEW = 1

    # How much one zoom click zooms in
    ZOOM_FACTOR = 0.5
    MOVE_SIZE = 100

    def __init__(self, ui, data):
        """Create the layout of the screen

        @param data: List [current photo index, List of Image objects]
        """
        self.config = Configuration()
        Screen.__init__(self)

        self.ui = ui
        self.animate = self.config.show_effects()
        self.slideshow_step = self.config.get_slideshow_step()

        self.view = self.PHOTO_VIEW             # Current view
        self.zoom_level = 1                     # Current zoom level
        self.index = data[0]                    # Index of the current image
        self.images = data[1]                   # List of image objects
        self.slideshow = False                  # Current state of the slideshow
        self.slideshow_timeout_tag = None       # Slideshow gobect timeout tag

        # Create black background
        self.background = clutter.Rectangle()
        self.background.set_size(self.ui.STAGE_WIDTH, self.ui.STAGE_HEIGHT)
        self.background.set_color((0, 0, 0, 255))
        self.background.show()
        self.add(self.background)

        # Screen Title (Displayed at the bottom left corner)
        self.screen_title = Label(0.13, "screentitle", 0, 0.87,
            _("Information"))
        self.add(self.screen_title)

        self.info_actors = None
        self.info = None
        self._create_information_view()

        self.texture = None
        self._change_image(self.index)

        self.show()

    def set_animate(self, boolean):
        """Animate this screen."""
        self.animate = boolean

    def _create_information_view(self):
        """
        Create actors for information view.
        """
        self.info = clutter.Group()
        text_size = 0.0416

        # Title
        title_h = Label(text_size, "subtitle", 0, 0, _("Title:"))
        self.info.add(title_h)

        title_text = self.images[self.index].get_title()
        title = Label(text_size, "title", 0.1464, 0, title_text)
        self.info.add(title)

        # Comment
        comment_h = Label(text_size, "subtitle", 0, 0.0911, _("Comment:"))
        self.info.add(comment_h)

        comment_text = self.images[self.index].get_description()
        comment = Label(text_size, "title", 0.1464, 0.0911, comment_text)
        self.info.add(comment)

        # Album
        album_h = Label(text_size, "subtitle", 0, 0.1823, _("Album:"))
        self.info.add(album_h)

        album_text = self.images[self.index].get_album().get_title()
        album = Label(text_size, "title", 0.1464, 0.1823, album_text)
        self.info.add(album)

        # Date
        date_h = Label(text_size, "subtitle", 0, 0.2709, _("Date:"))
        self.info.add(date_h)

        date_text = self.images[self.index].get_date()
        date = Label(text_size, "title", 0.1464, 0.2709, date_text)
        self.info.add(date)

        # Time
        time_h = Label(text_size, "subtitle", 0, 0.3646, _("Time:"))
        self.info.add(time_h)

        time_text = self.images[self.index].get_time()[:5]
        time = Label(text_size, "title", 0.1464, 03646, time_text)
        self.info.add(time)

        # Resolution
        resolution_h = Label(text_size, "subtitle", 0, 0.4557, _("Resolution:"))
        self.info.add(resolution_h)

        resolution_text = str(self.images[self.index].get_width()) + "x" + \
            str(self.images[self.index].get_height())
        resolution = Label(text_size, "title", 0.1464, 0.4557, resolution_text)
        self.info.add(resolution)

        # File size
        size_h = Label(text_size, "subtitle", 0, 0.5469, _("File size:"))
        self.info.add(size_h)

        size_text = str(self.images[self.index].get_filesize())
        size = Label(text_size, "title", 0.1464, 0.5469, size_text)
        self.info.add(size)

        # Create hashtable for items
        self.info_actors = {}
        self.info_actors['title'] = title
        self.info_actors['comment'] = comment
        self.info_actors['album'] = album
        self.info_actors['date'] = date
        self.info_actors['time'] = time
        self.info_actors['resolution'] = resolution
        self.info_actors['size'] = size

        self.info.set_position(self.ui.x(0.0732), self.ui.y(0.1302))
        self.info.set_opacity(0)
        self.add(self.info)

    def _update_information_view(self):
        """
        Update actors of the information view to correspond current image.
        """
        image = self.images[self.index]
        self.info_actors['title'].set_text(image.get_title())
        self.info_actors['comment'].set_text(image.get_description())
        self.info_actors['album'].set_text(image.get_album().get_title())
        self.info_actors['date'].set_text(image.get_date())
        self.info_actors['time'].set_text(image.get_time()[:5])
        self.info_actors['resolution'].set_text(str(image.get_width()) + \
            "x" + str(image.get_height()))
        self.info_actors['size'].set_text(str(image.get_filesize()))

    def _change_image(self, index):
        """
        Change current image. Display image from given index.
        """
        try:
            #self.remove(self.texture)
            #del self.texture
            self.texture.destroy()
        except AttributeError:
            pass # No previous texture in memory. No problem, let's continue

        # Create a new texture and display it
        image = self.images[index]
        self.index = index
        self.texture = Texture(image.get_filename())
        self._scale_image(self.texture)

        timeline = clutter.Timeline(fps=120, duration=1000)
        alpha = clutter.Alpha(timeline, clutter.smoothstep_inc_func)
        self.opacity_behaviour = clutter.BehaviourOpacity(alpha=alpha, 
            opacity_start=0, opacity_end=255)
        self.opacity_behaviour.apply(self.texture)
        self.texture.set_opacity(0)
        timeline.start()

        if self.view == self.INFORMATION_VIEW:
            self._scale_image(self.texture, 0.6)
            if self.texture.get_width() < self.texture.get_height():
                x_pos = ((self.ui.STAGE_WIDTH / 2 - self.texture.get_width() * \
                    self.texture.get_scale()[0]) / 2) + \
                    (self.ui.STAGE_WIDTH / 2)
            else:
                x_pos = (self.ui.STAGE_WIDTH / 2) 
            self.texture.set_position(int(x_pos), self.ui.y(0.1302))

        self._update_information_view()
        self.add(self.texture)

    def _scale_image(self, texture, zoom_level=1):
        """
        Scale image. Scaleing doesn't change image aspect ratio.
        @param texture: Texture to scale
        @param zoom_level: Zoom level - Default value is 1 (no zoom)
        """
        # TODO: Zoom feature is not implemented completely!
        # Center position when zoomed
        width = texture.get_width()
        height = texture.get_height()
        x_ratio = self.ui.STAGE_WIDTH / float(width)
        y_ratio = self.ui.STAGE_HEIGHT / float(height)

        if x_ratio > y_ratio:
            texture.set_anchor_point_from_gravity(clutter.GRAVITY_CENTER)
            texture.set_scale(self.ui.STAGE_HEIGHT / float(height) * zoom_level,
                              self.ui.STAGE_HEIGHT / float(height) * zoom_level)
            texture.set_anchor_point(0, 0)

            if zoom_level == 1: # Center image if in normal size
                new_width = int(width * (self.ui.STAGE_HEIGHT / float(height)))
                new_x = int((self.ui.STAGE_WIDTH - new_width) / float(2))
                texture.set_position(new_x, 0)
        else:
            texture.set_anchor_point_from_gravity(clutter.GRAVITY_CENTER)
            texture.set_scale(self.ui.STAGE_WIDTH / float(width) * zoom_level, 
                              self.ui.STAGE_WIDTH / float(width) * zoom_level)
            texture.set_anchor_point(0, 0)

            if zoom_level == 1:  # Center image if in normal size
                new_height = int(height * (self.ui.STAGE_WIDTH / float(width)))
                new_y = int((self.ui.STAGE_HEIGHT - new_height) / float(2))
                texture.set_position(0, new_y)

    def _calculate_photo_coordinates(self):
        """
        Return x position of the photograph. This X-coordinate sets photo
        to the center of the screen in fullscreen mode. This method is used
        when photo is animated from the info view to the fullscreen view.
        """
        width = self.texture.get_width()
        height = self.texture.get_height()
        x_ratio = self.ui.STAGE_WIDTH / float(width)
        y_ratio = self.ui.STAGE_HEIGHT / float(height)

        if x_ratio > y_ratio:
            new_width = int(width * (self.ui.STAGE_HEIGHT / float(height)))
            x_pos = int((self.ui.STAGE_WIDTH - new_width) / float(2))
            y_pos = 0
        else:
            new_height = int(height * (self.ui.STAGE_WIDTH / float(width)))
            y_pos = int((self.ui.STAGE_HEIGHT - new_height) / float(2))
            x_pos = 0
        return (x_pos, y_pos)

    def _change_to_info_view(self):
        """
        Change from fullscreen photo view to information view
        """
        # Calculate X-coordinate for small photograph
        if self.texture.get_width() < self.texture.get_height():
            x_pos = ((self.ui.STAGE_WIDTH / 2 - self.texture.get_width() * \
                self.texture.get_scale()[0]) / 2) + (self.ui.STAGE_WIDTH / 2)
        else:
            x_pos = (self.ui.STAGE_WIDTH / 2)

#        if self.animate:
#            # Fade in information texts and screen title
#            self.info_tl_1 = clutter.Timeline(13, 26)
#            alpha = clutter.Alpha(self.info_tl_1, clutter.ramp_inc_func)
#            behaviour = clutter.BehaviourOpacity(alpha, 0, 255)
#            behaviour.apply(self.screen_title)
#            behaviour.apply(self.info)
#
#            # Fade out black background
#            self.info_tl_2 = clutter.Timeline(13, 26)
#            alpha = clutter.Alpha(self.info_tl_2, clutter.ramp_inc_func)
#            behaviour = clutter.BehaviourOpacity(alpha, 255, 0)
#            behaviour.apply(self.background)
#
#            # Scale and move photograph to right side of the screen
#            self.info_tl_3 = clutter.Timeline(13, 26)
#            alpha = clutter.Alpha(self.info_tl_3, clutter.ramp_inc_func)
#            behaviour = clutter.BehaviourPath(alpha, 
#                ((self.texture.get_x(), self.texture.get_y()), (x,100)))
#            behaviour = clutter.BehaviourScale(alpha, 
#                self.texture.get_scale()[0], self.texture.get_scale()[0] * 0.6)
#            behaviour.apply(self.texture)
#
#            self.info_tl_1.start()
#            self.info_tl_2.start()
#            self.info_tl_3.start()
#            self.info_tl_4.start()
#        else:
        self.screen_title.show()
        self.background.set_opacity(0)
        self.info.set_opacity(255)
        self._scale_image(self.texture, 0.6)
        self.texture.set_position(x_pos, self.ui.y(0.1302))

    def _change_to_photo_view(self):
        """
        Change from information view to fullscreen photo view
        """
        #if self.animate:
        # Fade out information texts and screen title
#        self.photo_tl_1 = clutter.Timeline(13, 26)
#        alpha = clutter.Alpha(self.photo_tl_1, clutter.ramp_inc_func)
#        behaviour = clutter.BehaviourOpacity(255, 0, alpha)
#        behaviour.apply(self.screen_title)
#        behaviour.apply(self.info)
#
#        # Fade in black background
#        self.photo_tl_2 = clutter.Timeline(13, 26)
#        alpha2 = clutter.Alpha(self.photo_tl_2, clutter.ramp_inc_func)
#        behaviour = clutter.BehaviourOpacity(0, 255, alpha2)
#        behaviour.apply(self.background)
#
#        # Move photograph to fullscreen
#        self.photo_tl_3 = clutter.Timeline(13, 26)
#        alpha3 = clutter.Alpha(self.photo_tl_3, clutter.ramp_inc_func)
#        behaviour = clutter.BehaviourPath( alpha3, ((self.texture.get_x(),
#           self.texture.get_y()), self._calculate_photo_coordinates()))
#FIXME: Calculate corrent position
#        behaviour.apply(self.texture)
#
#        # Scale photograph to fullscreen
#        self.photo_tl_4 = clutter.Timeline(13, 26)
#        alpha4 = clutter.Alpha(self.photo_tl_4, clutter.ramp_inc_func)
#        behaviour = clutter.BehaviourScale(alpha4,
#           self.texture.get_scale()[0], self.texture.get_scale()[0] / 0.6)
#        behaviour.apply(self.texture)
#
#        self.photo_tl_1.start()
#        self.photo_tl_2.start()
#        self.photo_tl_3.start()
#        self.photo_tl_4.start()
        #else:
        self.screen_title.hide()
        self.background.set_opacity(255)
        self.info.set_opacity(0)
        self._scale_image(self.texture)

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "PhotoScreen"

    def is_interested_in_play_action(self):
        """
        Override function from Screen class. See Screen class for
        better documentation.
        """
        return True

    def execute_play_action(self):
        """
        Override function from Screen class. See Screen class for
        better documentation.
        """
        if self.slideshow:
            self.stop_slideshow()
        else:
            self.start_slideshow()

    def slideshow_progress(self):
        """slideshow loop called every slideshow_step seconds"""
        # use of a fake key event so that all the *mode* management stuff
        # is done only once.
        if (self.ui.current.get_name() != "PhotoScreen"):
            return False
        elif ((self.view == self.PHOTO_VIEW) and (self.zoom_level == 1) and
            self.slideshow):
            self.ui.handle_user_event(UserEvent(UserEvent.NAVIGATE_RIGHT))
        return True

    def start_slideshow(self):
        """Start the slideshow"""
        self.zoom_level = 1
        self.slideshow = True
        self.slideshow_progress()
        self.slideshow_timeout_tag = gobject.timeout_add(self.slideshow_step * 
            1000, self.slideshow_progress)

    def stop_slideshow(self):
        """Stop the slideshow"""
        self.slideshow = False
        if self.slideshow_timeout_tag:
            gobject.source_remove(self.slideshow_timeout_tag)

    def handle_user_event(self, event):
        """ 
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        if event_type == UserEvent.NAVIGATE_LEFT:
            # Change to previous image
            if self.zoom_level == 1 or self.view == self.INFORMATION_VIEW:
                if self.index == 0:
                    self.index = len(self.images) - 1
                else:
                    self.index = self.index - 1
                self._change_image(self.index)
            # Change texture position (texture is zoomed)
            else:
                self.texture.move_by(self.MOVE_SIZE, 0)

        elif event_type == UserEvent.NAVIGATE_RIGHT:
            # Change to next image
            if self.zoom_level == 1  or self.view == self.INFORMATION_VIEW:
                if self.index == len(self.images) - 1:
                    self.index = 0
                else:
                    self.index = self.index + 1
                self._change_image(self.index)
            # Change texture position (texture is zoomed)
            else:
                self.texture.move_by(-self.MOVE_SIZE, 0)

        elif event_type == UserEvent.NAVIGATE_UP and self.zoom_level != 1:
            self.texture.move_by(0, self.MOVE_SIZE)

        elif event_type == UserEvent.NAVIGATE_DOWN and self.zoom_level != 1:
            self.texture.move_by(0, -self.MOVE_SIZE)

        # Zoom image
        elif event_type == UserEvent.NAVIGATE_SELECT:
            # if we zoom then we stop sliding photos
            self.stop_slideshow()
            if self.view == self.PHOTO_VIEW:
                if self.zoom_level >= 3:
                    self.zoom_level = 1
                else:
                    self.zoom_level = self.zoom_level + self.ZOOM_FACTOR
                self._scale_image(self.texture, self.zoom_level)

        # Switch view (fullscreen photo / information screen)
        elif event_type == UserEvent.NAVIGATE_CONTENT:
        # XXX: laymansterms - This screen is totally half baked. There isn't
        # even a key that maps to NAVIGATE_CONTENT
            if self.view == self.PHOTO_VIEW:
                self.view = self.INFORMATION_VIEW
                self._change_to_info_view()
            else:
                self.view = self.PHOTO_VIEW
                self._change_to_photo_view()

