'''PhotoAlbumsScreen - Screen contains a list of photograph albums'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import gtk
import pango
import gobject
import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.transitions.transition import Transition
from entertainerlib.frontend.gui.widgets.eyecandy_texture import (
    EyeCandyTexture)
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.configuration import Configuration

class PhotoAlbumsScreen(Screen):
    """
    PhotoAlbumsScreen

    This screen contains a list of photo albums. This screen is used for
    selecting album from album list. Screen also displays preview for every
    album.
    """

    def __init__(self, ui, image_library):
        """Create the layout of the screen"""

        self.config = Configuration()
        Screen.__init__(self)

        self.ui = ui
        self.theme = self.config.theme
        self.library = image_library
        self.timeout = None # Timeout key (this is used when scrolling menu)

        # Screen Title (Displayed at the bottom left corner)
        screen_title = Label(0.13, "screentitle", 0, 0.87, _("Photographs"))
        self.add(screen_title)

        if self.library.get_number_of_images() == 0:
            self._create_no_photos_information()
        else:
            # Album preview group
            self.preview = clutter.Group()
            self.preview.set_position(self.ui.x(0.07), self.ui.y(0.1953))
            self.preview.show()
            self.add(self.preview)

            self.preview_fade = None
            self.menu = None
            self.in_behaviour = None
            self.out_behaviour = None
            self.in_opacity = None
            self.out_opacity = None
            self.preview_textures = None
            self._create_album_menu()
            self.li = None
            self._create_list_indicator()

        self.show()

    def _create_no_photos_information(self):
        """
        Create textures and labels for information screen. This is displayed
        instead of album list if there are no photos available and it helps
        users to add new photographs to the system.
        """
        # Create warning icon
        warning_icon = Texture(self.theme.getImage("warning_icon"), 0.28, 0.27)
        self.add(warning_icon)

        # Create warning title
        info_title = Label(0.0625, "title", 0.3367, 0.2709,
            _("No photographs available!"))
        self.add(info_title)

        # Create warning help text
        message = _(
            "There are no indexed photographs in the Entertainer media "
            "library. To add photographs, start the Content management tool "
            "and open the 'Images' tab. Now click on the 'Add' button and "
            "select some folders which contain image files.")
        info = Label(0.0417, "menuitem_inactive", 0.2804, 0.45, message)
        info.set_size(0.5, 0.5859)
        self.add(info)

    def _create_album_menu(self):
        """
        Create ImageAlbum-menu. This menu contains list of albums. It also
        displays number of photographs per album.
        """
        self.menu = TextMenu(self.theme, self.config.show_effects())
        self.menu.set_row_count(1)
        self.menu.set_visible_column_count(7)
        self.menu.set_item_size(self.ui.x(0.4393), self.ui.y(0.0781))
        self.menu.set_position(self.ui.x(0.5271), self.ui.y(0.3385))
        self.menu.show()

        # Create menu items based on ImageLibrary
        albums = self.library.get_albums()
        for album in albums:
            nro_of_photos = album.get_number_of_images()
            if nro_of_photos == 0: # Skip empty albums
                continue
            else:
                nro_of_photos = str(nro_of_photos)
            item = TextMenuItem(0.4393, 0.0781, album.get_title(),
                nro_of_photos)
            item.set_userdata(album) #FIXME: Should we use URLs as KEYS?
            self.menu.add_actor(item)

        self.add(self.menu)
        self._update_album_preview(
            self.menu.get_current_menuitem().get_userdata())

    def _create_album_preview(self, album):
        """
        Create a clutter.Group that contains album preview actors.
        """
        group = clutter.Group()
        group.set_position(self.ui.x(0.07), self.ui.y(0.1953))

        rect = clutter.Rectangle()
        rect.set_size(self.ui.x(0.4026), self.ui.y(0.7161))
        rect.set_color(clutter.Color(255, 255, 255, 32))
        rect.show()
        #group.add(rect)

        # Preview images
        images = album.get_preview_images(3)
        self.preview_textures = []

        for image in images:
            pix_buffer = gtk.gdk.pixbuf_new_from_file(image.get_thumbnail_url())
            texture = EyeCandyTexture(pix_buffer, pix_buffer.get_width(),
                pix_buffer.get_height())
            texture.set_rotation(clutter.Y_AXIS, 25, 0, 0, 0)
            texture.set_opacity(0)

            # Resize and center preview texture
            max_w = self.ui.x(0.4026)
            max_h = self.ui.y(0.5599)
            width, height = texture.get_size()
            x_ratio = max_w / float(width)
            y_ratio = max_h / float(height)
            if x_ratio > y_ratio:
                texture.set_scale(max_h /float(height), max_h / float(height))
                new_width = int(width * (max_h / float(height)))
                new_x = int((max_w - new_width) / 2.0)
                texture.set_position(new_x, 0)
            else:
                texture.set_scale(max_w / float(width), max_w / float(width))
                new_height = int(height * (max_w/ float(width)))
                new_y = int((max_h - new_height) / 2.0)
                texture.set_position(0, new_y)

            texture.show()
            self.preview_textures.append(texture)
            group.add(texture)
        self.preview_textures[0].set_opacity(255)

        title = Label(0.03646, "title", 0.4649, 0, album.get_title(),
            font_weight="bold")
        title.width = 0.4758
        title.set_ellipsize(pango.ELLIPSIZE_END)
        group.add(title)

        desc = Label(0.026, "subtitle", 0.4649, 0.0521, album.get_description())
        desc.width = 0.4758
        group.add(desc)

        return group

    def _update_album_preview(self, album):
        """
        Update album preview. Display preview images from the current album.
        @param album: Currently selected album in menu
        """
        try:
            gobject.source_remove(self.preview_fade)
        except:
            pass # If there is no image switcher enabled that's ok

        new = self._create_album_preview(album)

        if self.config.show_effects():
            old = self.preview
            new.set_opacity(0)
            new.show()
            self.preview = new
            self.add(self.preview)

            #Fade out timeline
            timeline1 = clutter.Timeline(10, 26)
            alpha1 = clutter.Alpha(timeline1, clutter.smoothstep_inc_func)
            self.out_opacity = clutter.BehaviourOpacity(255, 0, alpha1)
            self.out_opacity.apply(old)
            #self.out_scale = clutter.BehaviourScale(self.alpha1, 1.0, 0.8,
            #   clutter.GRAVITY_CENTER)
            #self.out_scale.apply(old)
            #self.out_path = clutter.BehaviourPath(self.alpha1, 
            #    ((old.get_x(), old.get_y()),
            #    (old.get_x(), old.get_y() + 50)))
            #self.out_path.apply(old)

            timeline1.connect('completed', self._change_preview_timeline_ended,
                old)

            # Fade in timeline
            timeline2 = clutter.Timeline(10, 26)
            alpha2 = clutter.Alpha(timeline2, clutter.smoothstep_inc_func)
            self.in_opacity = clutter.BehaviourOpacity(0, 255, alpha2)
            self.in_opacity.apply(new)
            #self.in_scale = clutter.BehaviourScale(self.alpha2, 1.2, 1.0,
            #    clutter.GRAVITY_CENTER)
            #self.in_scale.apply(new)
            #self.in_path = clutter.BehaviourPath(self.alpha2,
            #    ((new.get_x(), new.get_y() - 50),
            #    (new.get_x(), new.get_y())))
            #self.in_path.apply(new)

            # Start animation
            timeline1.start()
            timeline2.start()
        else:
            # Do it without animation
            if self.preview is not None:
                self.remove(self.preview)
            self.preview = new
            #self.preview.rotate_y(20,0,0)
            self.add(self.preview)
            self.preview.show()

        if len(self.preview_textures)>1:
            self.preview_fade = gobject.timeout_add(6000,
                self._change_preview_image)

        return False # see gobject.timeout_add() doc

    def _change_preview_timeline_ended(self, timeline, group):
        """
        This is a callback function for preview updates. This is called when
        transition effect is finished. This mehtod removes old preview group
        from the stage.
        """
        # TODO: Figure out if/how timeline needs to be used
        self.remove(group)

    def _change_preview_image(self):
        """
        Run a timeline that crossfades preview images. This method is a callback
        that is called every 4 seconds.
        """
        if len(self.preview_textures)<=1:
            self.preview_textures[0].set_opacity(255)
        elif self.config.show_effects():
            #Fade out timeline
            fade_out = clutter.Timeline(40, 26)
            alpha_out = clutter.Alpha(fade_out, clutter.smoothstep_inc_func)
            self.out_behaviour = clutter.BehaviourOpacity(255, 0, alpha_out)
            self.out_behaviour.apply(self.preview_textures[0])

            # Fade in timeline
            fade_in = clutter.Timeline(40, 26)
            alpha_in = clutter.Alpha(fade_in, clutter.smoothstep_inc_func)
            self.in_behaviour = clutter.BehaviourOpacity(0, 255, alpha_in)
            self.in_behaviour.apply(self.preview_textures[1])

            # Start animation
            fade_out.start()
            fade_in.start()
        else:
            self.preview_textures[0].set_opacity(0)
            self.preview_textures[1].set_opacity(255)

        # Scroll images
        self.preview_textures = self.preview_textures[1:] + \
            self.preview_textures[:1]

        return True

    def _create_list_indicator(self):
        """
        Create list indicator for albums list.
        """
        self.li = ListIndicator(0.2, 0.045, ListIndicator.VERTICAL)
        self.li.set_position(self.ui.x(0.9297) - self.li.get_width(),
            self.ui.STAGE_HEIGHT - self.li.get_height() - self.ui.y(0.03906))
        self.li.set_delimiter(" | ")
        self.li.show()
        self.li.set_maximum(self.menu.get_number_of_items())
        self.li.set_current(1)
        self.add(self.li)

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "PhotoAlbumScreen"

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        if event_type == UserEvent.NAVIGATE_UP:
            self.menu.move(TextMenu.UP)
            self.li.set_current(self.menu.get_current_postition() + 1)
            # 500ms timeout before preview is updated (fast scrolling)
            if self.timeout is not None:
                gobject.source_remove(self.timeout) 
            self.timeout = gobject.timeout_add(500, self._update_album_preview,
                self.menu.get_current_menuitem().get_userdata())
            #self._update_album_preview(
            #   self.menu.get_current_menuitem().get_userdata())

        elif event_type == UserEvent.NAVIGATE_DOWN:
            self.menu.move(TextMenu.DOWN)
            self.li.set_current(self.menu.get_current_postition() + 1)
            # 500ms timeout before preview is updated (fast scrolling)
            if self.timeout is not None:
                gobject.source_remove(self.timeout) 
            self.timeout = gobject.timeout_add(500, self._update_album_preview,
                self.menu.get_current_menuitem().get_userdata())
            #self._update_album_preview(
            #   self.menu.get_current_menuitem().get_userdata())

        elif event_type == UserEvent.NAVIGATE_SELECT:
            album = self.menu.get_current_menuitem().get_userdata()
            userdata = [album.get_title(), album.get_images()]
            self.ui.changeScreen(self.ui.createScreen("photographs", userdata),
                Transition.FORWARD)

