'''PhotographsScreen - Screen contains a list/grid of photographs'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import pango
import clutter
import gobject

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.transitions.transition import Transition
from entertainerlib.frontend.gui.widgets.image_menu import ImageMenu
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.image_menu_item import ImageMenuItem
from entertainerlib.frontend.gui.widgets.loading_animation import (
    LoadingAnimation)
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.configuration import Configuration

class PhotographsScreen(Screen):
    """
    PhotographsScreen

    This screen displays a grid of photographs as thumbnails. User can select
    any photograph from this grid.
    """

    def __init__(self, ui, data):
        """Create the layout of the screen

        @param data: List [screen title string, List of Image objects]
        """
        self.config = Configuration()
        Screen.__init__(self)

        self.ui = ui
        self.theme = self.config.theme
        self.images = data[1]

        # Screen Title (Displayed at the bottom left corner)
        screen_title = Label(0.13, "screentitle", 0, 0.87, data[0])
        self.add(screen_title)

        # Image Title (over album list)
        self.image_title = Label(0.04167, "title", 0.0586, 0.7943, " ")
        self.image_title.set_ellipsize(pango.ELLIPSIZE_END)
        self.add(self.image_title)

        self.image_desc = Label(0.04167, "subtitle", 0.0586, 0.9115, " ")
        self.image_desc.set_line_wrap(True)
        self.image_desc.set_ellipsize(pango.ELLIPSIZE_END)
        self.add(self.image_desc)

        # Display throbber animation while loading photographs
        self.throbber = LoadingAnimation(40, clutter.Color(255, 255, 255, 255),
            clutter.Color(128, 128, 128, 128))
        self.throbber.set_position(self.ui.x(0.9), self.ui.y(0.9))
        self.throbber.show()
        self.add(self.throbber)

        # List indicator
        self.li = None
        self._create_list_indicator()

        # Create photomenu
        self.menu = ImageMenu(self.theme)
        self.menu.set_visible_column_count(6)
        self.menu.set_position(self.ui.x(0.0586), self.ui.y(0.1823))

        if self.config.show_effects():
            self.menu.set_animate(True)

        self.menu.show()
        self.add(self.menu)

        photos = self.images
        self.behaviour = None
        gobject.timeout_add(25, self._create_photo_menuitems, photos)

        self._update_image_info(0)

        self.show()

    def _create_photo_menuitems(self, photographs):
        """
        This function is called as async callback. It is also recursive
        function. This allows us load menu items in the background without
        blocking the user interface.
        """
        # Check if we should stop recursive calls (menu is complete)
        if len(photographs) > 0:
            photo = photographs[0]
            texture = Texture(photo.get_thumbnail_url())
            item = ImageMenuItem(0.1464, 0.75, texture)
            item.set_userdata(photo)
            self.menu.add_actor(item)
            # Recursive call, remove first artist from the list
            gobject.timeout_add(15, self._create_photo_menuitems,
                photographs[1:])
            return False
        else:
            if self.config.show_effects():
                timeline = clutter.Timeline(35, 26)
                alpha = clutter.Alpha(timeline, clutter.smoothstep_inc_func)
                self.behaviour = clutter.BehaviourOpacity(255, 0, alpha)
                self.behaviour.apply(self.throbber)
                timeline.connect('completed', self._remove_throbber, None)
                timeline.start()
            else:
                self.throbber.hide()
                self.remove(self.throbber)
                del self.throbber
            return False

    def _remove_throbber(self, timeline, actor):
        """
        Callback function for timeline that fades out photograph menu throbber.
        This is called when animation is finished and we are free to remove
        throbber from memory.
        @param timeline: clutter.Timeline object
        @param actor: clutter.Actor object
        """
        # TODO: Figure out if/how timeline and actor should be used
        self.throbber.hide()
        self.remove(self.throbber)
        del self.throbber

    def _update_image_info(self, number):
        """
        Update image information box.
        @param number: Number of image. This is index of the self.__images
        """
        image = self.images[number]
        name = image.get_title()
        desc = image.get_description()
        self.image_title.set_text(name)
        self.image_title.set_size(0.366, 0.04167)
        self.image_desc.set_text(desc)
        self.image_desc.set_size(0.366, 0.0911)

    def _create_list_indicator(self):
        """
        Create list indicator for albums list.
        """
        self.li = ListIndicator(0.18, 0.045, ListIndicator.HORIZONTAL)
        self.li.set_position(self.ui.x(0.9297) - self.li.get_width(),
            self.ui.y(0.7943))
        self.li.set_delimiter(" | ")
        self.li.show()
        self.li.set_maximum(len(self.images))
        self.li.set_current(1)
        self.add(self.li)

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "PhotographsScreen"

    def handle_user_event(self, event):   
        """ 
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        if event_type == UserEvent.NAVIGATE_LEFT:
            self.menu.move(ImageMenu.LEFT)
            self._update_image_info(self.menu.get_current_postition())
            self.li.set_current(self.menu.get_current_postition() + 1)
        elif event_type == UserEvent.NAVIGATE_RIGHT:
            self.menu.move(ImageMenu.RIGHT)
            self._update_image_info(self.menu.get_current_postition())
            self.li.set_current(self.menu.get_current_postition() + 1)
        elif event_type == UserEvent.NAVIGATE_UP:
            self.menu.move(ImageMenu.UP)
            self._update_image_info(self.menu.get_current_postition())
            self.li.set_current(self.menu.get_current_postition() + 1)
        elif event_type == UserEvent.NAVIGATE_DOWN:
            self.menu.move(ImageMenu.DOWN)
            self._update_image_info(self.menu.get_current_postition())
            self.li.set_current(self.menu.get_current_postition() + 1)
        elif event_type == UserEvent.NAVIGATE_SELECT:
            index = self.menu.get_current_postition()
            self.ui.changeScreen(self.ui.createScreen("photo",
                [index, self.images]), Transition.FORWARD)

