'''QuestionScreen - Screen displays a question label and specified buttons.'''

__licence__ = "GPLv2"
__copyright__ = "2008"
__author__ = "Teje Rodgers <contact@tejerodgers.com>"

import gtk
import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.transitions.transition import Transition
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.configuration import Configuration

class QuestionScreen(Screen):
    """
    Question screen

    This is displayed when the application needs to ask a close ended question
    to the user.
    """
    def __init__(self, ui, data):
        """Create the layout of the screen

        @param data: [0] = question [1] = answers [2] = callbacks (items may be
        null)
        """
        self.config = Configuration()
        Screen.__init__(self)

        self.ui = ui
        self.theme = self.config.theme

        self.question = data[0]
        self.answers = data[1]
        self.callbacks = data[2]

        self.menu = None
        self.question_label = None

        self.ask_question()
        self.display_answers()

        self.show()

    def display_answers(self):
        """
        Display answers

        Display a menu with answers on the screen
        """
        self.menu = TextMenu(self.theme, self.config.show_effects())
        self.menu.set_name("questionmenu")
        self.menu.set_row_count(1)
        self.menu.set_visible_column_count(7)
        self.menu.set_item_size(self.ui.x(0.4393), self.ui.y(0.0781))
        self.menu.set_position(self.ui.x(0.095), self.ui.y(0.5))
        self.menu.show()

        for answer in self.answers:
            item = TextMenuItem(0.4173, 0.0781, str(answer))
            item.set_userdata(str(answer))
            self.menu.add_actor(item)

        self.add(self.menu)

    def ask_question(self):
        """
        Ask Question

        Display the question text
        """

        #Label at the bottom
        screen_title = Label(0.13, "screentitle", 0, 0.87, _("Question"))
        self.add(screen_title)

        #Actual question label
        question = Label(0.04167, "text", 0.095, 0.13, str(self.question))
        self.add(question)

    def callback(self, selected_answer):
        """
        Callback

        Call the function associated with the selected answer
        """
        callback_loc = 0
        for answer in self.answers:
            if str(answer) == selected_answer:
                break
            callback_loc += 1 

        if callback_loc < len(self.callbacks):
            callback = self.callbacks[callback_loc]
            if callback != None:
                callback()

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return object name, to comply with clutter"""
        return str(self)

    def __str__(self):
        """Return screen name (human readble)"""
        return "QuestionScreen"

    def go_back(self):
        """Go back to previous screen"""
        screen = self.ui.history.get_screen()
        screen.update()
        self.ui.changeScreen(screen, Transition.BACKWARD)

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        if event_type == UserEvent.NAVIGATE_UP:
            self.menu.move(self.menu.UP)

        elif event_type == UserEvent.NAVIGATE_DOWN:
            self.menu.move(self.menu.DOWN)

        elif event_type == UserEvent.NAVIGATE_SELECT:
            self.callback(self.menu.get_current_menuitem().get_userdata())
            self.go_back()

        elif event_type == UserEvent.NAVIGATE_BACK:
            self.go_back()
