'''RssScreen - Screen displays RSS feeds'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import pango
import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.transitions.transition import Transition
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.menu import Menu
from entertainerlib.frontend.gui.widgets.selector import Selector
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.medialibrary.feeds import Feed
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.configuration import Configuration
from entertainerlib.utils.feed_utils import FeedEntryParser

class RssScreen(Screen):
    """
    RssScreen

    This screen displays RSS-feeds from Feed library. Screen contains menu
    that includes all feed titles and number of entries per feed.
    """

    def __init__(self, ui, feed_library):
        """Create the layout of the screen

        Read data from FeedLibrary and create menu of feeds.
        """
        self.config = Configuration()
        Screen.__init__(self)

        self.feed_order = 'UNREAD' # Feeds ordered by number of unread entries
        self.active_menu = "feed"
        self.ui = ui
        self.theme = self.config.theme
        self.library = feed_library

        # Screen Title (Displayed at the bottom left corner)
        screen_title = Label(0.13, "screentitle", 0, 0.87, _("Headlines"))
        self.add(screen_title)

        # RSS Icon
        icon = Texture(self.theme.getImage("rss_icon"), 0.05, 0.07)
        self.add(icon)

        # RSS Title
        title = Label(0.0833, "title", 0.13, 0.07, _("RSS Feeds"))
        self.add(title)

        #We set the menus to None here so we can check if they exist later on
        self.menu = None
        self.context_menu = None

        # Display information box if there are not feeds available
        if self.library.is_empty() or len(self.library.get_feeds()) == 0:
            self._create_no_feeds_information()
        else:
            # Menus
            self._create_context_menu()
            self._create_feed_menu()
            self.li = None
            self._create_feed_menu_indicator()
            self.menu.set_active(True)
            self.context_menu.set_active(False)

            # Feeds Title
            feeds_title = Label(0.04167, "title", 0.3953, 0.2, _("Feeds"))
            self.add(feeds_title)

            # Unread Title
            unread_title = Label(0.04167, "title", 0.8053, 0.2, _("Unread"))
            self.add(unread_title)

            # Feed description text
            desc_text = FeedEntryParser().strip_tags(
                Feed(self.menu.get_current_menuitem().get_userdata()
                ).get_description())
            self.description = Label(0.03646, "text", 0.3953, 0.9, desc_text)
            self.description.set_ellipsize(pango.ELLIPSIZE_END)
            self.description.set_use_markup(True)
            self.description.set_size(0.5088, 0.0651)
            self.add(self.description)

        self.show()

    def _create_feed_menu(self):
        """
        Create Feed-menu. This menu contains list of Feeds. It also displays
        number of unread entries per feed.
        """
        self.menu = TextMenu(self.theme, self.config.show_effects())
        self.menu.set_row_count(1)
        self.menu.set_visible_column_count(7)
        self.menu.set_item_size(self.ui.x(0.5124), self.ui.y(0.0781))
        self.menu.set_position(self.ui.x(0.3807), self.ui.y(0.2604))

        # Create menu items based on FeedLibrary
        feeds = self.library.get_feeds(self.feed_order)
        for feed in feeds:
            unread = feed.get_number_of_unread()
            if unread == 0:
                unread = None
            else:
                unread = str(unread)
            item = TextMenuItem(0.5124, 0.0781,
                FeedEntryParser().strip_tags(feed.get_title()),
                unread)
            item.set_userdata(feed.get_feed_url())
            self.menu.add_actor(item)

        self.menu.show()
        self.add(self.menu)

    def _create_feed_menu_indicator(self):
        """
        Create a ListIndicator for feed menu.
        """
        self.li = ListIndicator(0.2, 0.045, ListIndicator.VERTICAL)
        self.li.set_position(self.ui.x(0.8858) - self.li.get_width(),
            self.ui.x(0.4649))
        self.li.set_delimiter(" | ")
        self.li.show()
        self.li.set_maximum(self.menu.get_number_of_items())
        self.add(self.li)

    def _create_context_menu(self):
        """
        Create RSS-feed context menu and add it to the screen.
        This menu contains buttons for update and sorting.
        """
        select = Selector(Selector.LARGE, self.theme)
        self.context_menu = Menu()

        self.context_menu.set_loop(True)
        item1 = TextMenuItem(0.2196, 0.0781, _("Update feeds"))
        self.context_menu.add(item1)
        item2 = TextMenuItem(0.2196, 0.0781, _("Mark all as read"))
        self.context_menu.add(item2)
        item3 = TextMenuItem(0.2196, 0.0781, _("Sort by name"))
        self.context_menu.add(item3)
        item4 = TextMenuItem(0.2196, 0.0781, _("Sort by unread"))
        self.context_menu.add(item4)

        self.context_menu.set_selector(select)
        self.context_menu.set_position(self.ui.x(0.0732), self.ui.y(0.2604))
        self.context_menu.set_active(False)
        self.context_menu.show_all()
        self.add(self.context_menu)

    def _create_no_feeds_information(self):
        """
        Create textures and labels for information screen. This is displayed
        instead of feeds list if there are no feeds available. This screen helps
        users to add new feeds to the system.
        """
        # Create warning icon
        warning_icon = Texture(self.theme.getImage("warning_icon"), 0.28, 0.27)
        self.add(warning_icon)

        # Create warning title
        info_title = Label(0.0625, "title", 0.3367, 0.2709,
            _("No headlines available!"))
        self.add(info_title)

        # Create warning help text
        message = _(
            "To use headlines you have to add some RSS feeds with the Content "
            "management tool. To do this, start the Content management tool, "
            "open the 'RSS Feeds' tab and click the 'Add' button. Now you can "
            "write an RSS feed URL in the text entry. You can get RSS feed "
            "URLs from webpages that support RSS.")
        info = Label(0.0417, "menuitem_inactive", 0.2804, 0.45, message)
        info.set_size(0.5, 0.5208)
        self.add(info)

    def update(self):
        """
        Update screen widgets. This is called always when screen is poped from
        the screen history.
        """
        #remove current feed menu, create a new one and set it to active
        self.remove(self.menu)
        self._create_feed_menu()
        self.menu.set_active(True)

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "RssScreen"

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        #If the menus do not exist, there's no point in navigating
        if self.menu is None or self.context_menu is None:
            return
        
        event_type = event.get_type()

        if event_type == UserEvent.NAVIGATE_UP:
            if self.active_menu == "feed":
                self.menu.move(TextMenu.UP)
                self.li.set_current(self.menu.get_current_postition() + 1)
                feed = Feed(self.menu.get_current_menuitem().get_userdata())
                self.description.set_text(FeedEntryParser().strip_tags(
                    feed.get_description()))
                self.description.set_size(0.5088, 0.0651)
                #self.li.set_current(self.menu.get_current_postition() + 1)
            else:
                self.context_menu.scroll_up()

        elif event_type == UserEvent.NAVIGATE_DOWN:
            if self.active_menu == "feed":
                self.menu.move(TextMenu.DOWN)
                self.li.set_current(self.menu.get_current_postition() + 1)
                feed = Feed(self.menu.get_current_menuitem().get_userdata())
                self.description.set_text(FeedEntryParser().strip_tags(
                    feed.get_description()))
                self.description.set_size(0.5088, 0.0651)
                #self.li.set_current(self.menu.get_current_postition() + 1)
            else:
                self.context_menu.scroll_down()

        # Make context active
        elif event_type == UserEvent.NAVIGATE_LEFT:
            if self.active_menu == "feed":
                self.active_menu = "context"
                self.li.hide_position()
                self.menu.set_active(False)
                self.description.hide()
                self.context_menu.set_active(True)

        # Make feed menu active
        elif event_type == UserEvent.NAVIGATE_RIGHT:
            if self.active_menu == "context":
                self.active_menu = "feed"
                self.li.show_position()
                self.menu.set_active(True)
                self.description.show()
                self.context_menu.set_active(False)

        # Menuitem selected (ENTER PRESSED)
        elif event_type == UserEvent.NAVIGATE_SELECT:
            if self.active_menu == "context":
                index = self.context_menu.get_current_postition()
                if index == 0:
                    #send message to bus and update screen
                    self.library.request_feed_update()
                    self.update()
                elif index == 1:
                    #tell feedlibrary to mark all as read and then update screen
                    self.library.mark_all_as_read()
                    self.update()
                elif index == 2:
                    #sort feeds by title ASC and update screen
                    self.feed_order = 'TITLE'
                    self.update()
                elif index == 3:
                    #sort feeds by UNREAD DESC and update screen
                    self.feed_order = 'UNREAD'
                    self.update()
            else:
                # Feed selected from the menu. Change screen.
                feed = Feed(self.menu.get_current_menuitem().get_userdata())
                self.ui.changeScreen(self.ui.createScreen("feed", feed),
                                       Transition.FORWARD)

