'''TVSerieEpisodesScreen - Screen allows user to browse episodes of a season'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import gtk
import pango
import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.transitions.transition import Transition
from entertainerlib.frontend.gui.widgets.eyecandy_texture import (
    EyeCandyTexture)
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.scroll_area import ScrollArea
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.configuration import Configuration

class TVSerieEpisodesScreen(Screen):
    """
    TVSerieEpisodesScreen

    This screen contains list of all episodes of one specific season.
    """

    def __init__(self, ui, data):
        """Create the layout of the screen

        @param data: List [ TVSerie object, Season number ]
        """
        self.config = Configuration()
        Screen.__init__(self)

        self.ui = ui
        self.theme = self.config.theme
        self.serie = data[0]
        self.season = data[1]

        # Screen Title (Displayed at the bottom left corner)
        screen_title = Label(0.13, "screentitle", 0, 0.87,
            self.serie.get_title())
        self.add(screen_title)

        self.scroll_area = None
        self.episode_menu = None
        self.title = None
        self.thumb = None
        self._create_episode_menu()
        self._create_episode_info_box()

        #List indicator
        self.li = ListIndicator(0.2, 0.045, ListIndicator.VERTICAL)
        self.li.set_position(self.ui.STAGE_WIDTH - self.li.get_width() - \
            self.ui.x(0.06589), self.ui.STAGE_HEIGHT - self.ui.y(0.1953))
        self.li.set_delimiter(" | ")
        self.li.show()
        self.li.set_maximum(self.episode_menu.get_number_of_items())
        self.li.set_current(1)
        self.add(self.li)

        self.show()

    def _create_episode_menu(self):
        """
        Create a list of available seasons
        """
        self.episode_menu = TextMenu(self.theme, self.config.show_effects())
        self.episode_menu.set_row_count(1)
        #self.episode_menu.set_visible_column_count(10)
        self.episode_menu.set_item_size(self.ui.x(0.4393), self.ui.y(0.0781))
        self.episode_menu.set_position(self.ui.x(0.4978), self.ui.y(0.1563))

        # Create menu items based on MusicLibrary
        episodes = self.serie.get_episodes_from_season(self.season)
        for episode in episodes:
            item = TextMenuItem(0.4393, 0.0781,
                str(episode.get_episode_number()) + ". " + episode.get_title())
            item.set_userdata(episode)
            self.episode_menu.add_actor(item)

        self.episode_menu.show()
        self.episode_menu.set_active(True)
        self.add(self.episode_menu)

    def _create_thumbnail_texture(self):
        """
        Create a thumbnail texture. This is called as menu is scrolled
        """
        try:
            self.thumb.hide()
            del self.thumb
        except:
            pass # No problem, this is just the first time we call this method

        # Thumbnail. Use cover art if thumbnail doesn't exist
        menu_item = self.episode_menu.get_current_menuitem()
        thumbnail = menu_item.get_userdata().get_thumbnail_url()
        if(thumbnail is not None):
            pixbuf = gtk.gdk.pixbuf_new_from_file(thumbnail)
            thumb_width = self.ui.x(0.2928) #FIXME
            thumb_height = self.ui.y(0.2799)
            thumb_x = self.ui.x(0.05)
            thumb_y = self.ui.y(0.2)
        else:
            thumb_width = self.ui.x(0.1098)
            thumb_height = self.ui.y(0.2799)
            thumb_x = self.ui.x(0.20)
            thumb_y = self.ui.y(0.15)
            if(self.serie.has_cover_art()):
                pixbuf = gtk.gdk.pixbuf_new_from_file(
                    self.serie.get_cover_art_url())
            else:
                pixbuf = gtk.gdk.pixbuf_new_from_file(
                    self.theme.getImage("default_movie_art"))

        self.thumb = EyeCandyTexture(pixbuf, thumb_width, thumb_height)
        self.thumb.set_position(thumb_x, thumb_y)
        self.thumb.show()
        self.add(self.thumb)

    def _create_episode_info_box(self):
        """
        Create a texture that is displayed next to track list. This texture
        displays album cover art.
        """
        self._create_thumbnail_texture()

        # Title
        self.title = Label(0.04, "title", 0.05, 0.55,
            self.episode_menu.get_current_menuitem().get_userdata().get_title(),
            font_weight="bold")
        self.title.set_ellipsize(pango.ELLIPSIZE_END)
        self.title.width = 0.4
        self.add(self.title)

        # Plot
        plot = Label(0.029, "subtitle", 0, 0,
            self.episode_menu.get_current_menuitem().get_userdata().get_plot())
        plot.width = 0.4

        self.scroll_area = ScrollArea(0.4, 0.3516, 0.05, 0.63, plot)
        self.add(self.scroll_area)

    def _update_episode_info_box(self, episode):
        """
        Update episode infobox
        @param episode: Display information from this episode
        """
        self._create_thumbnail_texture()
        self.title.set_text(episode.get_title())
        self.title.width = 0.4

        plot = Label(0.029, "subtitle", 0, 0,
            self.episode_menu.get_current_menuitem().get_userdata().get_plot())
        plot.width = 0.4
        self.scroll_area.set_content(plot)

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "TVSerieEpisodesScreen"

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        if event_type == UserEvent.NAVIGATE_UP:
            if self.episode_menu.is_active():
                self.episode_menu.move(self.episode_menu.UP)
                self.li.set_current(
                    self.episode_menu.get_current_postition() + 1)
                self._update_episode_info_box(
                    self.episode_menu.get_current_menuitem().get_userdata())
            else:
                self.scroll_area.scroll_up()
        elif event_type == UserEvent.NAVIGATE_DOWN:
            if self.episode_menu.is_active():
                self.episode_menu.move(self.episode_menu.DOWN)
                self.li.set_current(
                    self.episode_menu.get_current_postition() + 1)
                self._update_episode_info_box(
                    self.episode_menu.get_current_menuitem().get_userdata())
            else:
                self.scroll_area.scroll_down()
        elif event_type == UserEvent.NAVIGATE_RIGHT:
            self.episode_menu.set_active(True)
            self.scroll_area.set_active(False)
        elif event_type == UserEvent.NAVIGATE_LEFT:
            self.episode_menu.set_active(False)
            self.scroll_area.set_active(True)
        elif event_type == UserEvent.NAVIGATE_FIRST_PAGE:
            self.scroll_area.scroll_to_top()
        elif event_type == UserEvent.NAVIGATE_LAST_PAGE:
            self.scroll_area.scroll_to_bottom()
        elif event_type == UserEvent.NAVIGATE_PREVIOUS_PAGE:
            self.scroll_area.scroll_page_up()
        elif event_type == UserEvent.NAVIGATE_NEXT_PAGE:
            self.scroll_area.scroll_page_down()
        elif event_type == UserEvent.NAVIGATE_SELECT:
            episode = self.episode_menu.get_current_menuitem().get_userdata()
            mediaplayer = self.ui.get_media_player()
            mediaplayer.set_media(episode)
            mediaplayer.play()
            self.ui.changeScreen(self.ui.createScreen("video_osd"),
                Transition.FORWARD)

