'''TVSerieScreen - Screen allows user to browse seasons of one TV-series'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import gtk
import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.transitions.transition import Transition
from entertainerlib.frontend.gui.widgets.eyecandy_texture import (
    EyeCandyTexture)
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.configuration import Configuration

class TVSerieScreen(Screen):
    """
    TVSerieScreen

    This screen contains all seasons of one TV-series.
    """

    def __init__(self, ui, tv_series):
        """Create the layout of the screen"""

        self.config = Configuration()
        Screen.__init__(self)

        self.ui = ui
        self.theme = self.config.theme
        self.series = tv_series

        # Screen Title (Displayed at the bottom left corner)
        screen_title = Label(0.13, "screentitle", 0, 0.87,
            self.series.get_title())
        self.add(screen_title)

        self.art = None
        self.season_menu = None
        self._create_series_cover_texture()
        self._create_season_menu()

        #List indicator
        self.li = ListIndicator(0.2, 0.045, ListIndicator.VERTICAL)
        self.li.set_position(self.ui.STAGE_WIDTH - self.li.get_width() - \
            self.ui.x(0.0659), self.ui.STAGE_HEIGHT - self.ui.y(0.1953))
        self.li.set_delimiter(" | ")
        self.li.show()
        self.li.set_maximum(self.season_menu.get_number_of_items())
        self.li.set_current(1)
        self.add(self.li)

        self.show()

    def _create_season_menu(self):
        """
        Create a list of available seasons
        """
        self.season_menu = TextMenu(self.theme, self.config.show_effects())
        self.season_menu.set_row_count(1)
        self.season_menu.set_item_size(self.ui.x(0.4393), self.ui.y(0.0781))
        self.season_menu.set_position(self.ui.x(0.4978), self.ui.y(0.1563))

        # Create menu items based on MusicLibrary
        seasons = self.series.get_seasons()
        seasons.sort()
        for season in seasons:
            item = TextMenuItem(0.4393, 0.0781,
                _("Season %(num)s") % {'num': season})
            item.set_userdata(season)
            self.season_menu.add_actor(item)

        self.season_menu.show()
        self.season_menu.set_active(True)
        self.add(self.season_menu)

    def _create_series_cover_texture(self):
        """
        Create a texture that is displayed next to track list. This texture
        displays album cover art.
        """
        if(self.series.has_cover_art()):
            pixbuf = gtk.gdk.pixbuf_new_from_file(
                self.series.get_cover_art_url())
        else:
            pixbuf = gtk.gdk.pixbuf_new_from_file(
                self.theme.getImage("default_movie_art"))
        self.art = EyeCandyTexture(pixbuf, self.ui.x(0.2196), self.ui.y(0.5859))
        self.art.set_position(self.ui.x(0.16), self.ui.y(0.15))
        self.art.show()
        self.add(self.art)

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "TVSerieScreen"

    def handle_user_event(self, event):   
        """ 
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        if event_type == UserEvent.NAVIGATE_UP:
            self.season_menu.move(self.season_menu.UP)
            self.li.set_current(self.season_menu.get_current_postition() + 1)

        elif event_type == UserEvent.NAVIGATE_DOWN:
            self.season_menu.move(self.season_menu.DOWN)
            self.li.set_current(self.season_menu.get_current_postition() + 1)

        elif event_type == UserEvent.NAVIGATE_SELECT:
            season = self.season_menu.get_current_menuitem().get_userdata()
            data = [self.series, season]
            self.ui.changeScreen(self.ui.createScreen("tv-episodes", data),
                Transition.FORWARD)

