'''VideoOSDScreen - Screen is used for video playback OSD'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import gobject
import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.progress_bar import ProgressBar
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.user_event import UserEvent
from entertainerlib.utils.configuration import Configuration

class VideoOSDScreen(Screen):
    """
    VideoOSDScreen

    This screen is displayed when video is being watched. Usually this screen
    doesn't have any visible elements. User actions such as pause and rewind
    are displayed as on-screen-graphics.
    """

    def __init__(self, ui, player):
        """Create the layout of the screen"""

        self.config = Configuration()
        Screen.__init__(self)

        self.ui = ui
        self.theme = self.config.theme
        self.player = player

        self.progress_bar = None
        self.current_pos = None
        self.aspect_textures = None
        self.pause_texture = None
        self.seekbackward_texture = None
        self.seekforward_texture = None

        self.timeout_key = None
        self.progress_bar_timeout_key = None

        self._create_aspect_ratio_textures()

        self._create_navigation_textures()
        self._create_progress_bar()
        gobject.timeout_add(250, self._update_progress_bar) # 1/4 sec

    def _create_navigation_textures(self):
        '''Create the pause, seek-backward & seek-forward textures.'''
        self.pause_texture = Texture(
            self.theme.getImage("media-playback-pause"), 0.5, 0.5)
        self.pause_texture.set_anchor_point_from_gravity(
            clutter.GRAVITY_CENTER)
        self.pause_texture.hide()
        self.add(self.pause_texture)

        pause_timeline = clutter.Timeline(fps=120, duration=2000)
        pause_timeline.set_loop(True)
        alpha_pause = clutter.Alpha(pause_timeline, clutter.sine_func)

        self.pause_opacity = clutter.BehaviourOpacity(alpha=alpha_pause,
            opacity_start=0, opacity_end=255)
        self.pause_scale = clutter.BehaviourScale(1.0, 1.0, 1.3, 1.3,
            alpha_pause)

        self.pause_opacity.apply(self.pause_texture)
        self.pause_scale.apply(self.pause_texture)
        pause_timeline.start()

        self.seekbackward_texture = Texture(
            self.theme.getImage("media-seek-backward"), 0.1, 0.5)
        self.seekbackward_texture.set_anchor_point_from_gravity(
            clutter.GRAVITY_CENTER)
        self.seekbackward_texture.set_opacity(0)
        self.add(self.seekbackward_texture)

        self.seekbackward_timeline = clutter.Timeline(fps=120, duration=1000)
        alpha_seekbackward = clutter.Alpha(self.seekbackward_timeline,
            clutter.smoothstep_inc_func)

        self.seekbackward_opacity = clutter.BehaviourOpacity(
            alpha=alpha_seekbackward, opacity_start=255, opacity_end=0)
        self.seekbackward_opacity.apply(self.seekbackward_texture)

        self.seekforward_texture = Texture(
            self.theme.getImage("media-seek-forward"), 0.9, 0.5)
        self.seekforward_texture.set_anchor_point_from_gravity(
            clutter.GRAVITY_CENTER)
        self.seekforward_texture.set_opacity(0)
        self.add(self.seekforward_texture)

        self.seekforward_timeline = clutter.Timeline(fps=120, duration=1000)
        alpha_seekforward = clutter.Alpha(self.seekforward_timeline,
            clutter.smoothstep_inc_func)

        self.seekforward_opacity = clutter.BehaviourOpacity(
            alpha=alpha_seekforward, opacity_start=255, opacity_end=0)
        self.seekforward_opacity.apply(self.seekforward_texture)

    def _create_progress_bar(self):
        """
        Create the progress bar
        @author Joshua Scotton
        """
        # Current position as time stamp
        self.current_pos = Label(0.0365, "title", 0.6, 0.9, "00:00")
        self.add(self.current_pos)

        self.progress_bar = ProgressBar(self.ui.x(0.3001),
            self.ui.y(0.03906), self.theme.getColor("title"))
        self.progress_bar.set_name("progress_bar")
        self.progress_bar.set_position(self.ui.x(0.68), self.ui.y(0.901))
        self.add(self.progress_bar)
        self.display_progress_bar()

    def _update_progress_bar(self):
        """
        Update progress bar
        @author Joshua Scotton
        """
        self.progress_bar.set_progress(self.player.get_media_position())
        self.current_pos.set_text(self.player.get_media_position_string())
        return True

    def display_progress_bar(self, hide_after_delay=True):
        """
        This displays the progress bar for three seconds
        @author Joshua Scotton
        """
        self.progress_bar.show()
        self.current_pos.show()
        if self.progress_bar_timeout_key is not None:
            gobject.source_remove(self.progress_bar_timeout_key)
        if hide_after_delay:
            self.progress_bar_timeout_key = gobject.timeout_add(3000,
                self.hide_progress_bar)

    def hide_progress_bar(self):
        """
        This hides the progress bar
        @author Joshua Scotton
        """
        self.progress_bar.hide()
        self.current_pos.hide()

    def _create_aspect_ratio_textures(self):
        """
        Create textures that which are displayed when user changes aspect ratio.
        """
        texture_1 = Texture(self.theme.getImage("native_aspect_ratio"))
        texture_1.hide()
        self.add(texture_1)
        texture_2 = Texture(self.theme.getImage("widescreen_aspect_ratio"))
        texture_2.hide()
        self.add(texture_2)
        texture_3 = Texture(self.theme.getImage("zoom_aspect_ratio"))
        texture_3.hide()
        self.add(texture_3)
        texture_4 = Texture(self.theme.getImage("compromise_aspect_ratio"))
        texture_4.hide()
        self.add(texture_4)
        self.aspect_textures = [texture_1, texture_2, texture_3, texture_4]
        self.timeout_key = None # This is used when canceling timeouts
        for texture in self.aspect_textures:
            texture.position = (
                float(self.config.get_stage_width() - texture.get_width()) /
                (self.config.get_stage_width() * 2), 0.67)

    def _hide_aspect_ratio_logo(self, number):
        """
        Hide aspect ratio texture. This is a callback function and
        shouldn't be called directly.
        """
        self.aspect_textures[number].hide()
        self.timeout_key = None
        return False

    def _display_aspect_ratio_logo(self, number):
        """
        Display aspect ratio logo on screen when ratio is changed.
        @param number: Number of the aspect ratio
        """
        if self.timeout_key is not None:
            gobject.source_remove(self.timeout_key)
        for texture in self.aspect_textures:
            texture.hide()
        self.aspect_textures[number].show()
        self.timeout_key = gobject.timeout_add(2000,
            self._hide_aspect_ratio_logo, number)

    def get_type(self):
        """
        Get screen type.
        @return: Integer (See Screen class constants)
        """
        return Screen.OSD

    def get_name(self):
        """
        Get screen name (human readble)
        @return: string
        """
        return "VideoOSD"

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        event_type = event.get_type()

        if event_type == UserEvent.PLAYER_STOP:
            self.display_progress_bar(hide_after_delay=False)
            self.pause_texture.hide()
            #FIXME
            print "CREATE VIDEO STOPPED SCREEN HERE"

        # Video aspect ratio changed. Display aspect ratio logo on screen
        elif event_type == UserEvent.USE_ASPECT_RATIO_1:
            self._display_aspect_ratio_logo(0)
        elif event_type == UserEvent.USE_ASPECT_RATIO_2:
            self._display_aspect_ratio_logo(1)
        elif event_type == UserEvent.USE_ASPECT_RATIO_3:
            self._display_aspect_ratio_logo(2)
        elif event_type == UserEvent.USE_ASPECT_RATIO_4:
            self._display_aspect_ratio_logo(3)
        elif event_type == UserEvent.PLAYER_SKIP_FORWARD:
            self.display_progress_bar()
            self.seekforward_timeline.start()
        elif event_type == UserEvent.PLAYER_SKIP_BACKWARD:
            self.display_progress_bar()
            self.seekbackward_timeline.start()
        elif event_type == UserEvent.PLAYER_PLAY_PAUSE:
            if self.player.is_playing():
                self.pause_texture.hide()
                self.display_progress_bar()
            else:
                self.pause_texture.show()
                self.display_progress_bar(hide_after_delay=False)

