'''Tab - Tab is group of objects that are displayed on one tab'''

__licence__ = "GPLv2"
__copyright__ = "2008, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter

from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.utils.configuration import Configuration

class Tab(clutter.Group):
    """
    Tab can be used as part of the TabGroup

    Tab is a very simple container that contains all the widgets and logic
    of the tab page.
    """

    def __init__(self, ui, title="Untitled tab"):
        """Initialize tab"""

        clutter.Group.__init__(self)
        self.config = Configuration()
        self.title = title
        self.active = False
        self.theme = self.config.theme
        self.ui = ui

        self.timeline = clutter.Timeline(15, 60)
        self.alpha = clutter.Alpha(self.timeline, clutter.smoothstep_inc_func)
        self.behaviour = clutter.BehaviourOpacity(0, 255, self.alpha)
        self.behaviour.apply(self)

    def can_activate(self):
        """
        Returns a boolean whether or not this tab can be activated. If tab
        doesn't have any controlling widgets it shouldn't activate ever.
        Default value is False. Override this method when implementing a new
        tab and add logic if needed.
        """
        return False

    def is_active(self):
        """
        Is this tab active
        @return: boolean
        """
        return self.active

    def set_active(self, boolean):
        """
        Set this tab to active or inactive state
        @param boolean: True to set active, False to set inactive
        """
        self.active = boolean

    def show_empty_tab_notice(self, title=_("Empty tab"),
            message_body=_("This tab doesn't contain any elements.")):
        """
        Create an information box that is displayed if there is no widgets in
        this tab. This method should be called only from child class as needed.
        """

        # Create warning icon
        info_icon = Texture(self.theme.getImage("warning_icon"), 0.28, 0.27)
        self.add(info_icon)

        # Create warning title
        info_title = Label(0.0625, "title", 0.33, 0.27, title)
        self.add(info_title)

        # Create warning help text
        info = Label(0.042, "menuitem_inactive", 0.28, 0.4, message_body)
        info.width = 0.57
        self.add(info)

    def set_title(self, title):
        """
        Set title of the tab. This is displayed in Tab bar
        @param title: Title string
        """
        self.title = title

    def get_title(self):
        """
        Get title of the tab
        @return: Title string
        """
        return self.title

    def show_tab(self):
        """
        Activate this tab. This is called when tab is changed and this tab
        is becoming active. This method should activate animations.
        """
        self.set_opacity(0)
        self.show()
        self.timeline.set_direction(clutter.TIMELINE_FORWARD)
        self.timeline.start()

    def hide_tab(self):
        """
        Deactivate this tab. This is called when tab is changed and this tab
        is becoming non-active. This method should disable animations.
        """
        self.set_opacity(0)
        self.hide()

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        @return: Boolean, Should control be moved back to tab bar from tab
        """
        pass

