'''Tests for all models'''

__license__ = 'GPLv2'
__copyright__ = '2008, Paul Hummer'
__author__ = 'Paul Hummer <paul.hummer@canonical.com>'

import unittest
import sys
import os

from storm.locals import create_database, Store

from entertainerlib.backend.core.db import models
from entertainerlib.backend.core.db.connection import Database
from entertainerlib.tests import TestWithDatabase


class ModelTestCase(TestWithDatabase):
    '''Abstract Model Test Case

    The parent class of all model test classes.
    '''

    def setUp(self):
        self.init()
        # XXX: rockstar - Once the new database is set up, we'll want to have
        # this class automatically create the database
        self.db = Database(self.filename)

    def tearDown(self):
        pass


class TestNewsFeed(ModelTestCase):
    '''NewsFeed test case'''

    def setUp(self):
        ModelTestCase.setUp(self)

    def testCreate(self):
        '''Test creation of new NewsFeed'''

        store = Store(self.db)

        newsfeed = models.NewsFeed()
        newsfeed.url = u'http://theironlion.net/blog/feed'
        newsfeed.title = u'The Iron Lion'
        newsfeed.description = u'Paul Hummer\'s Blog'
        store.add(newsfeed)
        store.commit()

        self.assertTrue(Store.of(newsfeed) is store)

        newsfeed_from_database = store.find(models.NewsFeed,
            models.NewsFeed.title == u'The Iron Lion').one()

        self.assertTrue(newsfeed is newsfeed_from_database)


class TestNewsEntry(ModelTestCase):
    '''NewsEntry test case'''

    def setUp(self):
        ModelTestCase.setUp(self)

    def testCreate(self):
        '''Test creation of new NewsEntry'''

        store = Store(self.db)
        newsentry = models.NewsEntry()
        newsentry.url = u'http://theironlion.net/blog/foo-bar-baz'
        newsentry.title = u'Foo Bar Baz'
        newsentry.description = u'There once was a girl from Nantucket'
        newsentry.isread = 0
        store.add(newsentry)
        store.commit()

        self.assertTrue(Store.of(newsentry) is store)

        newsentry_from_database = store.find(models.NewsEntry,
            models.NewsEntry.title == u'Foo Bar Baz').one()
        self.assertTrue(newsentry is newsentry_from_database)


class TestPhotoAlbum(ModelTestCase):
    '''PhotoAlbum test case'''

    def setUp(self):
        ModelTestCase.setUp(self)

    def testCreate(self):
        '''Test creation of new PhotoAlbum'''

        store = Store(self.db)
        photoalbum = models.PhotoAlbum()
        photoalbum.title = u'Photo Title'
        photoalbum.description = u'This is a photo description'
        store.add(photoalbum)
        store.commit()

        self.assertTrue(Store.of(photoalbum) is store)

        photoalbum_from_database = store.find(models.PhotoAlbum,
            models.PhotoAlbum.title == u'Photo Title').one()
        self.assertTrue(photoalbum is photoalbum_from_database)


class TestPhotoImage(ModelTestCase):
    '''PhotoImage test case'''

    def setUp(self):
        ModelTestCase.setUp(self)

    def testCreate(self):
        '''Test creation of new PhotoImage'''

        store = Store(self.db)
        photoimage = models.PhotoImage()
        photoimage.filename = u'/home/user/photo.jpg'
        store.add(photoimage)
        store.commit()

        self.assertTrue(Store.of(photoimage) is store)

        photoimage_from_database = store.find(models.PhotoImage,
            models.PhotoImage.filename == u'/home/user/photo.jpg').one()
        self.assertTrue(photoimage is photoimage_from_database)


class TestMusicAlbum(ModelTestCase):
    '''MusicAlbum test case'''

    def setUp(self):
        ModelTestCase.setUp(self)

    def testCreate(self):
        '''Test creation of new MusicAlbum'''

        store = Store(self.db)
        musicalbum = models.MusicAlbum()
        musicalbum.title = u'The Lady Dance'
        store.add(musicalbum)
        store.commit()

        self.assertTrue(Store.of(musicalbum) is store)

        musicalbum_from_database = store.find(models.MusicAlbum,
            models.MusicAlbum.title == u'The Lady Dance').one()
        self.assertTrue(musicalbum is musicalbum_from_database)


class TestMusicTrack(ModelTestCase):
    '''MusicTrack test case'''

    def setUp(self):
        ModelTestCase.setUp(self)

    def testCreate(self):
        '''Test creation of new MusicTrack'''

        store = Store(self.db)
        musictrack = models.MusicTrack()
        musictrack.title = u'The Beautiful Ones'
        store.add(musictrack)
        store.commit()

        self.assertTrue(Store.of(musictrack) is store)

        musictrack_from_database = store.find(models.MusicTrack,
            models.MusicTrack.title == u'The Beautiful Ones').one()
        self.assertTrue(musictrack is musictrack_from_database)


class TestMusicPlaylist(ModelTestCase):
    '''MusicPlaylist test case'''

    def setUp(self):
        ModelTestCase.setUp(self)

    def testCreate(self):
        '''Test creation of new MusicPlaylist'''

        store = Store(self.db)
        musicplaylist = models.MusicPlaylist()
        musicplaylist.title = u'The Ultimate Heavy Metal Goth Emo'
        store.add(musicplaylist)
        store.commit()

        self.assertTrue(Store.of(musicplaylist) is store)

        musicplaylist_from_database = store.find(models.MusicPlaylist,
            models.MusicPlaylist.title == \
                u'The Ultimate Heavy Metal Goth Emo').one()
        self.assertTrue(musicplaylist is musicplaylist_from_database)


class TestVideoFile(ModelTestCase):
    '''VideoFile test case'''

    def setUp(self):
        ModelTestCase.setUp(self)

    def testCreate(self):
        '''Test creation of new VideoFile'''

        store = Store(self.db)
        videofile = models.VideoFile()
        videofile.filename = u'/home/user/foo.mpg'
        store.add(videofile)
        store.commit()

        self.assertTrue(Store.of(videofile) is store)

        videofile_from_database = store.find(models.VideoFile,
            models.VideoFile.filename == u'/home/user/foo.mpg').one()
        self.assertTrue(videofile is videofile_from_database)

