'''Configuration - Class represents Entertainer's configuration'''

__licence__ = "GPLv2"
__copyright__ = ("2007, Lauri Taimila", "2008, Matt Layman")
__author__ = ("Lauri Taimila <lauri@taimila.com>",
              "Matt Layman <laymansterms.dev@gmail.com>")

import os
import sys
import shutil
import ConfigParser
from ConfigParser import ParsingError, NoSectionError, NoOptionError

from entertainerlib.backend.core.message_type_priority import MessageType
from entertainerlib.backend.core.message_handler import MessageHandler

from entertainerlib.utils.theme import Theme

SOURCE_CONFIG = {
    'branch' : os.path.abspath(os.path.dirname(__file__) + '/../../cfg'),
    # Hardcoded path for a package install
    'package' : "/usr/local/share/entertainer/cfg"
    }

class Configuration(MessageHandler):
    """
    Configuration of the Entertainer

    This class is an interface to all configuration parameters. All components
    of Entertainer should get configuration values through this class. Object
    from this class reads current config files and returns values based on
    those files. If there are missing lines or other errors in config file,
    then object return default values defined in this class.
    """

    CFG_DIR = os.path.expanduser("~/.config/entertainer")

    _shared_state = {}

    def __init__(self, cfg_dir=CFG_DIR):
        """
        Read configuration files and setup this object.
        """
        self.__dict__ = self._shared_state
        MessageHandler.__init__(self)

        # cfg_dir inequalities are needed in this if statement for testing
        if not self._shared_state or (
            (cfg_dir != self.CFG_DIR) and (cfg_dir is not None)):
            self.cfg_dir = cfg_dir

            if not os.path.exists(os.path.expanduser(self.cfg_dir)):
                self.create_cfg_dir()

            self.ENTERTAINER_LOG = os.path.join(cfg_dir, u'entertainer.log')

            self.FEED_DB = os.path.join(cfg_dir, 'cache/feed.db')
            self.IMAGE_DB = os.path.join(cfg_dir, 'cache/image.db')
            self.MUSIC_DB = os.path.join(cfg_dir, 'cache/music.db')
            self.VIDEO_DB = os.path.join(cfg_dir, 'cache/video.db')

            self.IMAGE_THUMB_DIR = os.path.join(cfg_dir,
                'cache/thumbnails/image')
            self.VIDEO_THUMB_DIR = os.path.join(cfg_dir,
                'cache/thumbnails/video')
            self.ALBUM_ART_DIR = os.path.join(cfg_dir, 'cache/album_art')
            self.MOVIE_ART_DIR = os.path.join(cfg_dir, 'cache/movie_art')

            # Preferences file
            self.preferences_conf = os.path.join(cfg_dir, 'preferences.conf')

            # Content file
            self.content_conf = os.path.join(cfg_dir, 'content.conf')

            self.content_config = ConfigParser.ConfigParser()
            self.preferences = ConfigParser.ConfigParser()
            try:
                self.content_config.readfp(open(self.content_conf))
                self.preferences.readfp(open(self.preferences_conf))
            except ParsingError:
                print("ParsingError with configuration file.")
                sys.exit(1)
            except IOError:
                print("IOError: Couldn't read configuration file.")
                sys.exit(1)

            self.theme = Theme(self.get_theme_path())

            self.stage_width = None
            self.stage_height = None
            self.vertical_offset = None

    def create_cfg_dir(self):
        '''
        Create a configuration directory and default config files.
        '''
        config_dir = self.get_cfg_dir()
        try:
            # Copy configuration data from a dev branch if we can
            if os.path.exists(SOURCE_CONFIG['branch']):
                shutil.copytree(SOURCE_CONFIG['branch'], config_dir)
            # Install configuration data from the hardcoded package path
            elif os.path.exists(SOURCE_CONFIG['package']):
                shutil.copytree(SOURCE_CONFIG['package'], config_dir)
            else:
                print "Couldn't find configuration data. Execution aborted."
                sys.exit(1)
        except OSError:
            print "Couldn't copy configuration data to %s. Execution aborted." \
                % config_dir
            sys.exit(1)

    def update_configuration(self):
        """
        Read configuration files again and update this object.
        """
        try:
            self.content_config.readfp(open(self.content_conf))
            self.preferences.readfp(open(self.preferences_conf))
        except ParsingError:
            raise Exception("Syntax error in configuration file.")
        except IOError:
            raise IOError("Couldn't read config file.")

    def write_content_value(self, section, option, value):
        """Write a new value to the content configuration file."""
        try:
            self.content_config.set(section, option, value)
            cfg_file = file(self.content_conf, 'w')
            self.content_config.write(cfg_file)
        except NoSectionError:
            raise Exception("No Section to set in content.conf file")
        except NoOptionError:
            raise Exception("No Option to set in content.conf file")

    def write_preference_value(self, section, option, value):
        """Write a new value to the preferences configuration file."""
        try:
            self.preferences.set(section, option, value)
            cfg_file = file(self.preferences_conf, 'w')
            self.preferences.write(cfg_file)
        except NoSectionError:
            raise Exception("No Section to set in preferences.conf file")
        except NoOptionError:
            raise Exception("No Option to set in preferences.conf file")

    def get_stage_width(self):
        '''Get the stage width from the preferences'''
        if self.stage_width:
            return self.stage_width

        try:
            self.stage_width = self.preferences.getint("General", "stage_width")
        except (NoSectionError, NoOptionError):
            self.stage_width = 1366 # Default

        return self.stage_width

    def set_stage_width(self, new_width):
        '''Set the stage width for the in memory instance'''
        self.stage_width = new_width

    def get_stage_height(self):
        '''Get the stage height from the preferences'''
        if self.stage_height:
            return self.stage_height

        try:
            self.stage_height = self.preferences.getint("General",
                "stage_height")
        except (NoSectionError, NoOptionError):
            self.stage_height = 768 # Default

        return self.stage_height

    def set_stage_height(self, new_height):
        '''Set the stage height for the in memory instance'''
        self.stage_height = new_height

    def get_vertical_offset(self):
        '''Get the vertical offset that will keep widgets within a 16:9 ratio'''
        self.vertical_offset = int((self.get_stage_height() -
            (self.get_stage_width() * .5625)) / 2)
        return self.vertical_offset

    def get_theme_path(self):
        """
        Get absolute path of the used theme.
        @return: string (path)
        """
        theme_path = os.path.join(self.get_cfg_dir(), 'themes')
        try:
            theme = self.preferences.get("General", "theme")
        except (NoSectionError, NoOptionError):
            return os.path.join(theme_path, "Default")

        if os.path.exists(os.path.join(theme_path, theme)):
            return os.path.join(theme_path, theme)
        else:
            return os.path.join(theme_path, "Default")

    def is_notifications_enabled(self):
        """
        Display on-screen notification bubbles on events.
        @return: boolean
        """
        try:
            result = self.preferences.getboolean("General",
                "display_notifications")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def tray_icon_enabled(self):
        """
        Display tray icon.
        @return: boolean
        """
        try:
            result = self.preferences.getboolean("General", "display_icon")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def get_feed_fetch_interval(self):
        """
        Get time interval (in minutes) for feed fetching.
        @return: Integer
        """
        try:
            result = self.content_config.getint("RSS", "fetch_interval")
        except (NoSectionError, NoOptionError):
            return 60 # Default is one hour
        return result

    def get_port(self):
        """
        Get backend server's port number.
        @return: Integer
        """
        try:
            result = self.preferences.getint("General", "backend_port")
        except (NoSectionError, NoOptionError):
            return 45054 # Default port
        return result

    def get_video_folders(self):
        """
        Get list of video folders
        @return:String Array
        """
        #FIXME: We should prevent ENTERTAINER_DIR/cache/* to be in those folders
        #       This can lead to recursivnes: make thumbs of thumbs of thumbs...
        try:
            folder_list = self.content_config.get("Videos", "folders")
            result = folder_list.split(';')
        except (NoSectionError, NoOptionError):
            return []
        return result

    def get_music_folders(self):
        """
        Get list of music folders
        @return: String Array
        """
        #FIXME: We should prevent ENTERTAINER_DIR/cache/* to be in those folders
        #       This can lead to recursivnes: make thumbs of thumbs of thumbs...
        try:
            music_list = self.content_config.get("Music", "folders")
            result = music_list.split(';')
        except (NoSectionError, NoOptionError):
            return []
        return result

    def get_image_folders(self):
        """
        Get list of image folders
        @return: String Array
        """
        #FIXME: We should prevent ENTERTAINER_DIR/cache/* to be in those folders
        #       This can lead to recursivnes: make thumbs of thumbs of thumbs...
        try:
            image_list = self.content_config.get("Images", "folders")
            result = image_list.split(';')
        except (NoSectionError, NoOptionError):
            return []
        return result

    def get_feeds(self):
        """
        Get list of feeds.
        @return: String Array
        """
        try:
            rss_feeds = self.content_config.get("RSS", "feeds")
            result = rss_feeds.split(';')
        except (NoSectionError, NoOptionError):
            return []
        return result

    def get_weather_location(self):
        """
        Get list of weather location codes.
        @return: List of strings
        """
        try:
            location = self.content_config.get("Weather", "location")
        except (NoSectionError, NoOptionError):
            location = ''
        return location

    def get_weather_metric(self):
        """
        Should we use metric units when displaying weather conditions
        @return: Boolean
        """
        try:
            result = self.content_config.getboolean("Weather", "metric_units")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def display_weather_in_frontend(self):
        """
        Should we display weather in frontend
        @return: Boolean
        """
        try:
            result = self.content_config.getboolean("Weather",
                "display_in_menu")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def display_cd_eject_in_frontend(self):
        """
        Should we display the cd eject button in frontend
        @return: Boolean
        @author: Joshua Scotton
        """
        try:
            result = self.content_config.getboolean("CD",
                "display_eject_in_menu")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def download_video_metadata(self):
        """
        Get True if video metadata should be downloaded, otherwise False
        """
        try:
            result = self.content_config.getboolean("Videos",
                "download_metadata")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def download_album_art(self):
        """
        Download album art from the Internet.
        @return: boolean
        """
        try:
            result = self.content_config.getboolean("Music",
                "download_album_art")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def download_lyrics(self):
        """
        Download song lyrics from the Internet.
        @return: boolean
        """
        try:
            result = self.content_config.getboolean("Music", "download_lyrics")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def show_effects(self):
        """
        Use animations on user interface.
        @return: boolean
        """
        try:
            result = self.preferences.getboolean("General", "show_effects")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def transition_effect(self):
        """
        Which transition effect should be used when switching screens.
        @return: string (name of the effect)
        """
        try:
            result = self.preferences.get("General", "transition_effect")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def get_theme_name(self):
        """
        Get the name of the current theme.
        @return: string (theme name)
        """
        try:
            result = self.preferences.get("General", "theme")
        except (NoSectionError, NoOptionError):
            result = "Default"
        return result

    def start_in_fullscreen(self):
        """
        Return True if frontend should start in Fullscreen mode, otherwise False
        @return: boolean
        """
        try:
            result = self.preferences.getboolean("General",
                "start_in_fullscreen")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def start_auto_server(self):
        """
        Return True if backend server should auto start
        @return: boolean
        """
        try:
            result = self.preferences.getboolean("General",
                "start_server_auto")
        except (NoSectionError, NoOptionError):
            return False
        return result

    def history_size(self):
        """
        Get screen history size. This determines how many screen are kept in
        memory when scrolling.
        @return: Integer
        """
        try:
            result = self.preferences.getint("General", "history_size")
        except (NoSectionError, NoOptionError):
            return 10
        return result

    def get_slideshow_step(self):
        """
        Get the slideshow step in seconds used by the photo slideshow
        @return: Integer
        """
        try:
            result = self.preferences.getint("Photographs", "slideshow_step")
        except (NoSectionError, NoOptionError):
            return 5
        return result

    # Implements MessageHandler interface
    def handleMessage(self, message):
        """
        Handle received messages.
        @param message: Received Message object
        """
        if message.get_type() == MessageType.CONTENT_CONF_UPDATED:
            self.update_configuration()
        elif message.get_type() == MessageType.PREFERENCES_CONF_UPDATED:
            self.update_configuration()

    def get_cfg_dir(self):
        """
        Get the entertainer configuration dir.
        @return: strings
        """
        return self.cfg_dir

    def set_cfg_dir(self, dir):
        """
        Set the entertainer configuration dir.
        """
        if os.path.exists(os.path.expanduser(dir)):
            self.cfg_dir = os.path.expanduser(dir)
        else:
            print "Trying to set non-existent configuration directory"

