'''SystemTrayIcon class - Implements system tray icon for Entertainer.'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import os
import sys

import gobject
import pygtk
import gtk
import gtk.glade
import pynotify

from entertainerlib.backend.backend_server import BackendServer
from entertainerlib.backend.core.message import Message
from entertainerlib.backend.core.message_bus_proxy import MessageBusProxy
from entertainerlib.backend.core.message_type_priority import (
    MessagePriority, MessageType)
from entertainerlib.backend.core.message_handler import MessageHandler
from entertainerlib.utils.log_viewer import LogViewer
from entertainerlib.utils.preferences_dialog import PreferencesDialog
from entertainerlib.utils.content_management_dialog import (
    ContentManagementDialog)
from entertainerlib.utils.configuration import Configuration
from entertainerlib.frontend.frontend_client import FrontendClient

class SystemTrayIcon:
    """Implements system tray icon for entertainer."""

    FILE_DIR = os.path.dirname(__file__)
    GLADE_DIR = os.path.join(FILE_DIR, "glade")

    # Icon states
    NORMAL_ICON = 0
    RECORD_ICON = 1

    def __init__(self, debug=False, start_frontend=False, frontend=None,
        notify=True):
        """
        Create system tray icon and pop-up menu for it.
        @author ["Joshua Scotton", "Lauri Taimila"]
        @param debug If True than debugging inotify messages will be shown
        @param start_frontend should the frontend be started
        @param frontend FrontendClient to be associated with this class
        @param notify Should messages be shown? defaults to True
        """
        self.debug = debug
        self.config = Configuration()
        self.popup = None
        if not (frontend == None):
            self.frontend = frontend
            self.frontend_visible = True
        else:
            self.frontend = None
            self.frontend_visible = False
        self.notify = notify

        # Path to the tray icon when using a branch
        self.tray_icon_url = os.path.join(self.FILE_DIR, "..", "..", "icons",
            "hicolor", "24x24", "apps", "entertainer.png")

        self.icon_widget = gtk.StatusIcon()
        self.icon_widget.set_tooltip(_("Entertainer Server"))

        # Check if running from a branch to set the tray icon
        if (os.path.exists(self.tray_icon_url)):
            self.icon_widget.set_from_file(self.tray_icon_url)
        else:
            # Must be running from a package, therefore available by icon name
            self.icon_widget.set_from_icon_name("entertainer")

        self.message_handler = SystemTrayMessageHandler(self, debug)

        # Load glade files
        self.menu_widgets = gtk.glade.XML(
            os.path.join(self.GLADE_DIR, "system_tray_icon_menu.glade"))

        # Bind menu signals
        callback_dic = {"on_menuitem_frontend_activate"
                        : self.on_menuitem_frontend_activate,
                        "on_menuitem_preferences_activate"
                        : self.on_menuitem_preferences_activate,
                        "on_menuitem_content_management_activate"
                        : self.on_menuitem_content_management_activate,
                        "on_menuitem_log_viewer_activate"
                        : self.on_menuitem_log_viewer_activate,
                        "on_menuitem_quit_activate"
                        : self.on_menuitem_quit_activate
                        }
        self.menu_widgets.signal_autoconnect(callback_dic)
        self.popup = self.menu_widgets.get_widget("SystemTrayIconMenu")
        self.icon_widget.connect('activate', self.systray_icon_activated)
        self.icon_widget.connect('popup-menu', self.open_popup_menu)
        if(start_frontend):
            self.set_frontend_visible(True)

    def show_message(self, message,
                    title=_("Entertainer Media Center"),
                    length=5, icon="dialog-information",
                    urgency="LOW"):
        """
        Show libnotify message
        Thanks to http://roscidus.com/desktop/node/336 for info on this
        @author Joshua Scotton
        """
        if self.notify:
            if pynotify.init("Entertainer Media Center"):
                notifier = pynotify.Notification(title, message, icon)
                if urgency == "CRITICAL":
                    notifier.set_urgency(pynotify.URGENCY_CRITICAL)
                elif urgency == "NORMAL":
                    notifier.set_urgency(pynotify.URGENCY_NORMAL)
                else:
                    notifier.set_urgency(pynotify.URGENCY_LOW)
                notifier.set_timeout(length*1000)
                notifier.attach_to_status_icon(self.icon_widget)
                notifier.show()
            else:
                raise Exception("Problem with pynotify")
        else:
            print message

    def show_icon(self):
        """Show icon in system tray"""
        self.icon_widget.set_visible(True)

    def hide_icon(self):
        """Hide icon from system tray"""
        self.icon_widget.set_visible(False)

    def change_icon(self, icon_state):
        """Change displayed icon. RECORD_ICON is used when recording TV."""
        gtk.gdk.threads_enter()
        if icon_state == self.NORMAL_ICON:
            # Check if running from a branch to set the tray icon
            if (os.path.exists(self.tray_icon_url)):
                self.icon_widget.set_from_file(self.tray_icon_url)
            else:
                # Must be running from a package, therefore available by icon
                # name
                self.icon_widget.set_from_icon_name("entertainer")
            self.icon_widget.set_tooltip(_("Entertainer Server"))
        elif icon_state == self.RECORD_ICON:
            self.icon_widget.set_from_icon_name("gtk-media-record")
            self.icon_widget.set_tooltip(_("Entertainer is recording TV"))
        gtk.gdk.threads_leave()

    def systray_icon_activated(self, widget, data= None):
        """Switch visibility of frontend when system tray icon is clicked"""
        self.switch_frontend_visibility()

    def open_popup_menu(self, widget, button, time, data = None):
        """Display pop-up menu when system tray icon is clicked"""
        self.popup.show_all()
        self.popup.popup(None, None, None, 3, time)

    def on_menuitem_frontend_activate(self, widget):
        """Execute frontend here if not running. Show if running"""
        self.set_frontend_visible(True)

    def on_menuitem_preferences_activate(self, widget):
        """Executes preferences-tool."""
        if self.debug:
            self.show_message("Starting preferences...")
        PreferencesDialog(False)

    def on_menuitem_content_management_activate(self, widget):
        """Executes content-management-tool"""
        if self.debug:
            self.show_message("Starting content management...")
        ContentManagementDialog(False)

    def on_menuitem_log_viewer_activate(self, widget):
        """Display log viewer dialog"""
        if self.debug:
            self.show_message("Starting log viewer...")
        LogViewer(False)

    def on_menuitem_quit_activate(self, widget):
        """Show confirmation dialog and quit backend"""
        #FIXME Doesn't work yet hangs on threads_enter so commented out
        self.get_message_bus_proxy().disconnectFromMessageBus()
        if self.debug:
            self.show_message("Quitting...")

        if(not self.frontend == None):
            self.frontend.quit_frontend()
        #gtk.gdk.threads_enter()
        gtk.main_quit()
        #gtk.gdk.threads_leave()
        sys.exit()

    def set_message_bus_proxy(self, proxy):
        """
        This sets the MessageBusProxy object for the SystemTrayMessageHandler
        @author Joshua Scotton
        """
        self.get_message_handler().set_message_bus_proxy(proxy)

    def get_message_bus_proxy(self):
        """
        This gets the MessageBusProxy object from the SystemTrayMessageHandler
        @author Joshua Scotton
        """
        return self.get_message_handler().get_message_bus_proxy()

    def set_message_handler(self, message_handler):
        """
        This sets the associated SystemTrayMessageHandler
        @author Joshua Scotton
        """
        self.message_handler = message_handler

    def get_message_handler(self):
        """
        This gets the associated SystemTrayMessageHandler
        @author Joshua Scotton
        """
        return self.message_handler

    def set_frontend_visible(self, show=True):
        """
        Set the frontend to visible
        @author Joshua Scotton
        """
        if(show):
            if(self.frontend == None):
                if self.debug:
                    self.show_message("Starting Frontend...")
                self.frontend = FrontendClient(True)
            else:
                if self.debug:
                    self.show_message("Showing Frontend...")
                self.frontend.ui.get_stage().show()
            self.frontend_visible = True
        elif(not show and not self.frontend == None):
            if self.debug:
                self.show_message("Hiding Frontend...")
            self.frontend.ui.get_stage().hide()
            self.frontend_visible = False

    def get_frontend_visible(self):
        """
        Return the visibility of the frontend
        @author Joshua Scotton
        """
        return self.frontend_visible

    def switch_frontend_visibility(self):
        """
        Switches the visibility of the frontend
        @author Joshua Scotton
        """
        if(self.get_frontend_visible()==False):
            self.set_frontend_visible(True)
        else:
            self.set_frontend_visible(False)

class SystemTrayMessageHandler(MessageHandler):
    """
    This handles the messages which are intended for the system tray icon
    @author Joshua Scotton
    """

    def __init__(self, tray_icon, debug=False):
        """
        This sets up the SystemTrayMessageHandler object
        """
        MessageHandler.__init__(self)
        self.debug = debug
        self.tray_icon = tray_icon
        self.message_bus_proxy = None

    def set_tray_icon(self, tray_icon):
        """
        This sets the SystemTrayIcon object associated with this SystemTrayMessageHandler
        """
        self.tray_icon = tray_icon
    def get_tray_icon(self):
        """
        Gets the SystemTrayIcon for this object
        """
        return self.tray_icon
    def set_message_bus_proxy(self, proxy):
        """
        This sets the MessageBusProxy for this object
        """
        self.message_bus_proxy = proxy
    def get_message_bus_proxy(self):
        """
        This gets the MessageBusProxy for this object
        """
        return self.message_bus_proxy
    def set_debug(self, debug):
        """
        This sets debugging messages to True or False
        """
        self.debug = debug
    def get_debug(self):
        """
        Returns whether debugging messages should be shown
        """
        return self.debug

    # Action Methods Below
    def action_recording_started(self):
        """Handles RECORDING_STARTED message"""
        self.get_tray_icon().change_icon(self.get_tray_icon().RECORD_ICON)
        self.get_tray_icon().show_message(_("Recording started..."))
    def action_recording_stopped(self):
        """Handles RECORDING_STOPPED message"""
        self.get_tray_icon().change_icon(self.get_tray_icon().NORMAL_ICON)
        self.get_tray_icon().show_message(_("Recording stopped..."))
    def action_recording_conflict(self):
        """Handles RECORDING_CONFLICT message"""
        self.get_tray_icon().change_icon(self.get_tray_icon().NORMAL_ICON)
        self.get_tray_icon().show_message(_("Recording Conflict!"), 
                                          urgency="NORMAL", 
                                          icon="dialog-warning")

    # Implement MessageHandler interface

    def handleMessage(self, message):
        """Handles messages for the SystemTrayIcon"""

        #Simple dict switch statement
        {MessageType.RECORDING_STARTED: self.action_recording_started,
         MessageType.RECORDING_STOPPED: self.action_recording_stopped,
         MessageType.RECORDING_CONFLICT: self.action_recording_conflict
         }[message.get_type()]()

        if self.get_debug():
            self.get_tray_icon().show_message("Got message type: " + 
                                              str(message.get_type()))

def init_systray():
    '''Start the systray tasks.'''

    config = Configuration()
    message_dict = { MessageType.RECORDING_STARTED : MessagePriority.VERY_LOW,
                 MessageType.RECORDING_STOPPED : MessagePriority.VERY_LOW}

    gobject.threads_init()
    gtk.gdk.threads_init()

    # Create tray icon
    gtk.gdk.threads_enter()

    #Start SystemTrayIcon with show on start True
    notify = config.is_notifications_enabled()
    tray_icon = SystemTrayIcon(start_frontend=True, notify=notify)
    gtk.gdk.threads_leave()

    try:
        # Open connection to backend
        proxy = MessageBusProxy(message_dict, tray_icon.get_message_handler(),
            "system-tray-icon")
        proxy.connectToMessageBus()
        proxy.start()
        tray_icon.get_message_handler().set_message_bus_proxy(proxy)

    except:
        print "Backend is not running. Start backend and try again."
        sys.exit(1)

    gtk.gdk.threads_enter()
    gtk.main()
    gtk.gdk.threads_leave()

