"""
Weather Class
  Uses google weathers api to get a weather forecast for the next
  4 days.
"""
__licence__ = "GPLv2"
__copyright__ = ("2008, Jamie Bennett", "2008, Samuel Buffet")
__author__ = ("Jamie Bennett <jamie@linuxuk.org>", "Samuel Buffet")

import locale
import urllib
import urllib2
from xml.dom import minidom

from entertainerlib.utils.configuration import Configuration

DEFAULT_LOCALE = "en_US"

#The lookup url for the google weather API
WEATHER_LOOKUP_URL = "http://www.google.co.uk/ig/api?weather="

#use the icon instead of condition to make it work with any languages
WEATHER_IMAGES = {'/images/weather/mostly_sunny.gif': 'weather-sun-mostly',
                  '/images/weather/chance_of_rain.gif': 'weather-rain-chance',
                  #'/images/weather/partly_sunny.gif': 'weather-sun-clouds', 
                  # no dedicated gif provided by google for that weather 
                  # condition.
                  # it's a *mostly_sunny* gif wich is given instead :(
                  '/images/weather/rain.gif': 'weather-rain',
                  '/images/weather/storm.gif': 'weather-lightning',
                  '/images/weather/fog.gif': 'weather-fog',
                  '/images/weather/sunny.gif': 'weather-sun',
                  '/images/weather/icy.gif': 'weather-ice',
                  '/images/weather/cloudy.gif': 'weather-clouds',
                  '/images/weather/snow.gif': 'weather-snow'
                  }

class Weather:
    """
    Weather class
    @author: Jamie Bennett <jamie@linuxuk.org>
    """

    def __init__(self, location="", metric="", test_dir=None):
        """
        Sets up the weather object
        @param location (String) location of weather to get forecast
        for.
        """
        self.__location = ''
        self.__forecasts = []
        self.__metric = ''
        self.theme = Configuration(test_dir).theme
        self.set_location(location)
        (self.languagename, self.locale) = self.__get_locale ()
        self.refresh()

    def __get_locale (self):
        '''Get locale information from user\'s machine'''
        # adapted from *albumart_downloader.py*
        # list of supported languages by google weather API to be completed
        #
        # still remains a display issue with arabic, greek, hebrew, japanese...
        # wich is probably a Clutter issue. So let's wait Clutter 0.8 to see 
        # if it makes a difference
        supported_locales = {
            "en_US" : ("ENGLISH","en-us"),
            "ar" : ("ARABIC","ar"),
            "bg" : ("BULGARIAN","bg"),
            "zh" : ("CHINESE","zh"),
            "cs" : ("CZECH","cs"),
            "da" : ("DANISH","da"),
            "nl" : ("DUTCH","nl"),
            "et" : ("ESTONIAN","et"),
            "fi" : ("FINNISH","fi"),
            "fr" : ("FRENCH","fr"),
            "de" : ("GERMAN","de"),
            "el" : ("GREEK","el"),
            "he" : ("HEBREW","he"),
            "hi" : ("HINDI","hi"),
            "hu" : ("HUNGARIAN","hu"),
            "it" : ("ITALIAN","it"),
            "ja" : ("JAPANESE","ja"),
            "ko" : ("KOREAN","ko"),
            "pl" : ("POLISH","pl"),
            "pt" : ("PORTUGUESE","pt"),
            "ro" : ("ROMANIAN","ro"),
            "ru" : ("RUSSIAN","ru"),
            "hr" : ("CROATIAN","hr"),
            "sk" : ("SLOVAK","sk"),
            "sl" : ("SLOVENIAN","sl"),
            "es" : ("SPANISH","es"),
            "sv" : ("SWEDISH","sv"),
            "tr" : ("TURKISH","tr")
        }

        lc_id = DEFAULT_LOCALE
        default = locale.getdefaultlocale ()[0]
        if default:
            if supported_locales.has_key (default):
                lc_id = default
            else:
                lang = default.split("_")[0]
                if supported_locales.has_key (lang):
                    lc_id = lang

        return supported_locales[lc_id]


    def find_forecast(self, search):
        """
        Returns the search results page for the search
        @param search String
        @return array of forecasts
        """
        url = WEATHER_LOOKUP_URL + search
        url = url.replace(" ", "%20")

        txheaders = {
            'Accept-Language': self.locale
        }

        req = urllib2.Request(url, None, txheaders)
        rep = urllib2.urlopen(req)
        data = rep.read()

        # if this encoding is not specified for the xml file, parsing will
        # fail for some special characters
        data = data.replace('<?xml version="1.0"?>', 
            '<?xml version="1.0" encoding="iso-8859-1"?>')
       
        dom = minidom.parseString(data)

        in_forecast = 0
        day = ''
        low = ''
        high = ''
        condition = ''
        imagename = ''
        image = ''

        for node in dom.getElementsByTagNameNS('*', '*'):
            # a problem occured so bail out
            if (node.nodeName == 'problem_cause'):
                print _('Location not found or network problem')
                break

            if (node.nodeName == 'forecast_conditions'):
                in_forecast = 1

            if (in_forecast):
                if node.nodeName == 'day_of_week':
                    day = node.getAttribute('data')
                if node.nodeName == 'low':
                    low = node.getAttribute('data')
                    converted_low = self.weather_units (int(low))
                if node.nodeName == 'high':
                    high = node.getAttribute('data')
                    converted_high = self.weather_units (int(high))
                if node.nodeName == 'condition':
                    condition = node.getAttribute('data')
                if node.nodeName == 'icon':
                    imagename = node.getAttribute('data')
                    image = self.set_image (imagename)

            # condition is the last element of the forecast we are
            # interested in so write out the forecast if this is set.
                if (condition):
                    forecast = {"Day":day,
                                "Low":str(converted_low),
                                "High":str(converted_high),
                                "Condition":condition,
                                "Image":image}
                    self.add_forecast(forecast)
                    in_forecast = 0
                    day = ''
                    low = ''
                    high = ''
                    condition = ''
                    imagename = ''
                    image = ''

        return self.__forecasts

    def set_location(self, location):
        """
        Sets the current location
        @param type String should be a city, country name or
        post/zip code
        """
        self.__location = location

    def get_location(self):
        """
        Gets the current location
        @return type String of city, country name or post/zip code
        """
        return self.__location

    def set_metric(self, metric):
        """
        Sets the current metric flag
        @param type bool, true = use metric, false = imperial
        """
        self.__metric = metric

    def get_metric(self):
        """
        Gets the current metric flag
        @return type bool, true = use metric, false = imperial
        """
        return self.__metric

    def clear_forecasts(self):
        """
        Resets the forecast array
        """
        self.__forecasts = []

    def add_forecast(self, forecast):
        """
        Appends a forecast to the forecast array
        @param forecast dict
        """
        self.__forecasts.append(forecast)

    def get_forecasts(self):
        """
        Returns an array of forecasts
        @return array of forecasts
        """
        return self.__forecasts

    def weather_units (self, temp):
        """
        Convert (or not) a given imperial unit temp to metric depending
        on what the configuration file says to use for display units.
        @return type int converted (or not) weather temp
        """
        converted_temp = temp

        if (self.__metric):
            converted_temp = (temp-32) * (5.0/9)

        return int(round(converted_temp, 0))

    def set_image(self, condition):
        """
        Returns an image for a given weather condition.
        @return type String of weather image
        """
        try:
            image_name = WEATHER_IMAGES[condition]
        except KeyError:
            image_name = 'weather-na'

        image = self.theme.getImage(image_name)

        return image

    def refresh(self):
        """
        Clear current weather and forecasts and then loads new data
        from xml or rss feed
        """
        if (self.__location):
            self.clear_forecasts()
            self.find_forecast(self.__location)

